@@ -3,7 +3,7 @@
 #include "atray.h"
 #include "applet.h"
 #include "yprefs.h"
-#include "prefs.h"
+#include "default.h"
 #include "wmapp.h"
 #include "wmmgr.h"
 #include "wmframe.h"
@@ -20,8 +20,8 @@ static YColorName invisibleTaskBarAppFg(&clrInvisibleTaskBarAppText);
 static YColorName invisibleTaskBarAppBg(&clrInvisibleTaskBarApp);
 static YColorName groupingBg(&clrActiveTaskBarAppText);
 static YColorName groupingFg(&clrActiveTaskBarApp);
-static ref<YFont> normalTaskBarFont;
-static ref<YFont> activeTaskBarFont;
+static YFont normalTaskBarFont;
+static YFont activeTaskBarFont;
 
 TaskBarApp::TaskBarApp(ClientData* frame, TaskButton* button) :
     fFrame(frame),
@@ -509,9 +509,9 @@ void TaskButton::paint(Graphics& g, const YRect& r) {
     int textY = 0;
     mstring str(fActive ? fActive->getIconTitle() : null);
     if (str != null) {
-        ref<YFont> font = getFont();
+        YFont font = getFont();
         if (font != null) {
-                g.setColor(fg);
+            g.setColor(fg);
             g.setFont(font);
 
             int iconSize = 0;
@@ -525,7 +525,7 @@ void TaskButton::paint(Graphics& g, const YRect& r) {
                                (height() + font->height() -
                                (LOOK(lookMetal | lookFlat) ? 2 : 1)) / 2 -
                                font->descent());
-            int const wm = int(width()) - p - pad - iconSize - 1;
+            int const wm = int(width()) - p - pad - iconSize - 2;
 
             if (0 < wm && p + tx + wm < int(width())) {
                 textX = p + tx;
@@ -574,7 +574,7 @@ int TaskButton::estimate() {
 
     mstring str(fActive ? fActive->getIconTitle() : null);
     if (str != null) {
-        ref<YFont> font = getFont();
+        YFont font = getFont();
         if (font != null) {
             if (taskBarShowWindowIcons)
                 p += 2;
@@ -593,8 +593,8 @@ unsigned TaskButton::maxHeight() {
     return 2 + max(activeHeight, normalHeight);
 }
 
-ref<YFont> TaskButton::getFont() {
-    ref<YFont> font;
+YFont TaskButton::getFont() {
+    YFont font;
     if (fActive && getFrame()->focused())
         font = getActiveFont();
     if (font == null)
@@ -602,15 +602,15 @@ ref<YFont> TaskButton::getFont() {
     return font;
 }
 
-ref<YFont> TaskButton::getNormalFont() {
+YFont TaskButton::getNormalFont() {
     if (normalTaskBarFont == null)
-        normalTaskBarFont = YFont::getFont(XFA(normalTaskBarFontName));
+        normalTaskBarFont = normalTaskBarFontName;
     return normalTaskBarFont;
 }
 
-ref<YFont> TaskButton::getActiveFont() {
+YFont TaskButton::getActiveFont() {
     if (activeTaskBarFont == null)
-        activeTaskBarFont = YFont::getFont(XFA(activeTaskBarFontName));
+        activeTaskBarFont = activeTaskBarFontName;
     return activeTaskBarFont;
 }
 
@@ -800,7 +800,6 @@ TaskPane::TaskPane(IAppletContainer* taskBar, YWindow* parent):
     fForceImmediate(false),
     fTaskGrouping(taskBarTaskGrouping)
 {
-    addStyle(wsNoExpose);
     if (getGradient() == null && taskbackPixmap == null) {
         setBackground(taskBarBg);
     }
@@ -997,7 +996,7 @@ void TaskPane::relayoutNow(bool force) {
 
     int tc = 0;
 
-    for (IterTask task = fTasks.iterator(); ++task; ) {
+    for (TaskButton* task : fTasks) {
         if (task->getShown())
             tc++;
         else
@@ -1012,11 +1011,15 @@ void TaskPane::relayoutNow(bool force) {
     int x = 0;
     int lc = 0;
 
-    for (IterTask task = fTasks.iterator(); ++task; ) {
+    for (TaskButton* task : fTasks) {
         if (task->getShown()) {
             const int w1 = wid + (lc < rem);
             if (task != dragging()) {
-                task->setGeometry(YRect(x, 0, unsigned(w1), height()));
+                YRect r(x, 0, unsigned(w1), height());
+                if (rightToLeft) {
+                    r.xx = width() - r.xx - r.ww - 1;
+                }
+                task->setGeometry(r);
                 task->show();
             }
             x += w1;
@@ -1042,10 +1045,17 @@ void TaskPane::handleClick(const XButtonEvent& up, int count) {
     }
 }
 
+void TaskPane::handleExpose(const XExposeEvent& expose) {
+    YRect r(expose.x, expose.y, expose.width, expose.height);
+    paint(getGraphics(), r);
+}
+
 void TaskPane::paint(Graphics& g, const YRect& r) {
     if (taskbackPixmap == null && getGradient() == null) {
         g.setColor(taskBarBg);
         g.fillRect(r.x(), r.y(), r.width(), r.height());
+    } else {
+        clearArea(r.xx, r.yy, r.ww, r.hh);
     }
 }
 
