@@ -107,6 +107,9 @@ void YFrameWindow::snapTo(int &wx, int &wy) {
 
     int mx, my, Mx, My;
     manager->getWorkArea(this, &mx, &my, &Mx, &My, getScreen());
+    int vo = min(borderY(), int(topSideVerticalOffset));
+    if (my <= wy)
+        my -= vo;
 
     /// !!! clean this up, it should snap to the closest thing it finds
 
@@ -135,11 +138,11 @@ void YFrameWindow::snapTo(int &wx, int &wy) {
         xp += borderX();
         flags |= 8;
     }
-    if (yp < 0 || yp + height() > desktop->height()) {
+    if (yp < -vo || yp + height() > desktop->height()) {
         yp += borderY();
         flags |= 16;
     }
-    snapTo(xp, yp, 0, 0, desktop->width(), desktop->height(), flags);
+    snapTo(xp, yp, 0, -vo, desktop->width(), desktop->height(), flags);
     if (flags & 8) {
         xp -= borderX();
         flags &= ~8;
@@ -580,7 +583,7 @@ void YFrameWindow::manualPlace() {
     buttonDownY = 0;
 
     if (!xapp->grabEvents(desktop,
-                          YXApplication::movePointer,
+                          YWMApp::movePointer,
                           ButtonPressMask |
                           ButtonReleaseMask |
                           PointerMotionMask))
@@ -810,14 +813,7 @@ bool YFrameWindow::handleKey(const XKeyEvent &key) {
 }
 
 void YFrameWindow::constrainPositionByModifier(int &x, int &y, const XMotionEvent &motion) {
-    unsigned int mask = motion.state & (ShiftMask | ControlMask);
-
-    x += borderX();
-    y += borderY();
-    x -= borderX();
-    y -= borderY();
-
-    if (snapMove && !(mask & (ControlMask | ShiftMask))) {
+    if (snapMove && notbit(motion.state, ShiftMask | ControlMask)) {
         snapTo(x, y);
     }
 }
@@ -895,7 +891,7 @@ void YFrameWindow::startMoveSize(bool doMove, bool byMouse,
         buttonDownY = mouseYroot;
 
         wmapp->signalGuiEvent(geWindowMoved);
-        grabPointer = YXApplication::movePointer;
+        grabPointer = YWMApp::movePointer;
     } else if (!doMove) {
         wmapp->signalGuiEvent(geWindowSized);
 
@@ -919,7 +915,7 @@ void YFrameWindow::startMoveSize(bool doMove, bool byMouse,
             else if (grabX == 1)
                 grabPointer = YWMApp::sizeRightPointer;
             else
-                grabPointer = YXApplication::leftPointer;
+                grabPointer = YWMApp::leftPointer;
 
         }
 
@@ -977,12 +973,13 @@ void YFrameWindow::endMoveSize() {
     }
 }
 
-void YFrameWindow::handleBeginDrag(const XButtonEvent &down, const XMotionEvent &motion) {
+bool YFrameWindow::handleBeginDrag(const XButtonEvent &down, const XMotionEvent &motion) {
     if (down.button == Button3 && canMove()) {
         startMoveSize(true, true,
                       0, 0,
                       down.x, down.y);
         handleDrag(down, motion);
+        return true;
     }
     else if (down.button == Button1 && canSize()) {
         Window sw = (down.subwindow && indicatorsCreated)
@@ -1028,7 +1025,9 @@ void YFrameWindow::handleBeginDrag(const XButtonEvent &down, const XMotionEvent
                           grabX, grabY,
                           down.x_root, down.y_root);
         }
+        return true;
     }
+    return false;
 }
 
 void YFrameWindow::moveWindow(int newX, int newY) {
@@ -1078,22 +1077,24 @@ void YFrameWindow::handleMotion(const XMotionEvent &motion) {
         int newWidth = width(), newHeight = height();
 
         handleResizeMouse(motion, newX, newY, newWidth, newHeight);
-
-        drawMoveSizeFX(x(), y(), width(), height());
-        setCurrentGeometryOuter(YRect(newX, newY, newWidth, newHeight));
-        drawMoveSizeFX(x(), y(), width(), height());
-
+        YRect rect(newX, newY, newWidth, newHeight);
+        if (rect != geometry()) {
+            drawMoveSizeFX(x(), y(), width(), height());
+            setCurrentGeometryOuter(rect);
+            drawMoveSizeFX(x(), y(), width(), height());
+        }
         statusMoveSize->setStatus(this);
-        return ;
-    } else if (movingWindow) {
+    }
+    else if (movingWindow) {
         int newX = x();
         int newY = y();
 
         handleMoveMouse(motion, newX, newY);
         moveWindow(newX, newY);
-        return ;
     }
-    YWindow::handleMotion(motion);
+    else {
+        YWindow::handleMotion(motion);
+    }
 }
 
 
