@@ -103,6 +103,17 @@ enum EAction {
     actionLayerThirteen      = 237,
     actionLayerFullscreen    = 239,
     actionLayerAboveAll      = 241,
+
+    actionAboutClose         = 243,
+    actionTileLeft           = 245,
+    actionTileRight          = 246,
+    actionTileTop            = 247,
+    actionTileBottom         = 248,
+    actionTileTopLeft        = 249,
+    actionTileTopRight       = 250,
+    actionTileBottomLeft     = 251,
+    actionTileBottomRight    = 252,
+    actionTileCenter         = 253,
 };
 
 bool canShutdown(RebootShutdown reboot);
