@@ -1,17 +1,16 @@
-#ifndef __WMAPP_H
-#define __WMAPP_H
+#ifndef WMAPP_H
+#define WMAPP_H
 
 #include "ysmapp.h"
-#include "ymenu.h"
 #include "ymsgbox.h"
 #include "guievent.h"
+#include "yicon.h"
 
 class YWindowManager;
 class AboutDlg;
 class CtrlAltDelete;
-class SwitchWindow;
 
-enum FocusModels {
+enum FocusModel {
     FocusCustom,
     FocusClick,
     FocusSloppy,
@@ -25,7 +24,7 @@ class YSMListener {
 public:
     virtual void handleSMAction(WMAction message) = 0;
     virtual void restartClient(const char *path, char *const *args) = 0;
-    virtual int runProgram(const char *path, const char *const *args) = 0;
+    virtual int runProgram(const char *path, const char *const *args, int fd) = 0;
     virtual void runCommand(const char *prog) = 0;
     virtual void runOnce(const char *resource, long *pid,
                          const char *path, char *const *args) = 0;
@@ -55,7 +54,7 @@ public:
     virtual void handleSignal(int sig);
     virtual bool handleIdle();
     virtual bool filterEvent(const XEvent &xev);
-    virtual void actionPerformed(YAction action, unsigned int modifiers);
+    virtual void actionPerformed(YAction action, unsigned int modifiers = 0);
 
     virtual void handleMsgBox(YMsgBox *msgbox, int operation);
     virtual void handleSMAction(WMAction message);
@@ -69,13 +68,13 @@ public:
     virtual void smDie();
 #endif
 
-    void setFocusMode(FocusModels mode);
+    void setFocusMode(FocusModel mode);
     void initFocusMode();
     void initFocusCustom();
-    void loadFocusMode();
+    FocusModel loadFocusMode();
 
     virtual void restartClient(const char *path, char *const *args);
-    virtual int runProgram(const char *path, const char *const *args);
+    virtual int runProgram(const char *path, const char *const *args, int fd = -1);
     virtual void runOnce(const char *resource, long *pid,
                          const char *path, char *const *args);
     virtual void runCommand(const char *prog);
@@ -83,6 +82,9 @@ public:
     bool mapClientByPid(const char* resource, long pid);
     bool mapClientByResource(const char* resource, long *pid);
 
+    static YCursor leftPointer;
+    static YCursor rightPointer;
+    static YCursor movePointer;
     static YCursor sizeRightPointer;
     static YCursor sizeTopRightPointer;
     static YCursor sizeTopPointer;
@@ -96,17 +98,16 @@ public:
     static YCursor scrollUpPointer;
     static YCursor scrollDownPointer;
 
-    static ref<YIcon> getDefaultAppIcon();
+    ref<YIcon> getDefaultAppIcon();
 
-    bool hasCtrlAltDelete() const { return ctrlAltDelete != 0; }
+    bool hasCtrlAltDelete() const { return ctrlAltDelete != nullptr; }
     CtrlAltDelete* getCtrlAltDelete();
-    bool hasSwitchWindow() const { return switchWindow != 0; }
-    SwitchWindow* getSwitchWindow();
     const char* getConfigFile() const { return configFile; }
-    FocusModels getFocusMode() const { return focusMode; }
+    FocusModel getFocusMode() const { return focusMode; }
     YMenu* getWindowMenu();
-
+    void subdirs(const char* subdir, bool themeOnly, MStringArray& paths);
     void unregisterProtocols();
+    void refreshDesktop();
 
 private:
     char** mainArgv;
@@ -118,36 +119,42 @@ private:
     // XXX: these pointers are PITA because they can become wild when objects
     // are destroyed independently by manager. What we need is something like std::weak_ptr...
     YMsgBox *fLogoutMsgBox;
+    YMsgBox* fRestartMsgBox;
     AboutDlg* aboutDlg;
 
     CtrlAltDelete* ctrlAltDelete;
-    SwitchWindow* switchWindow;
     YMenu* windowMenu;
     int errorRequestCode;
     YFrameWindow* errorFrame;
     lazy<YTimer> errorTimer;
+    lazy<YTimer> pathsTimer;
     lazy<YTimer> splashTimer;
     lazy<YWindow> splashWindow;
+    lazy<GuiSignaler> guiSignaler;
 
     void createTaskBar();
     YWindow* splash(const char* splashFile);
+    virtual Cursor getRightPointer() const { return rightPointer; }
     virtual bool handleTimer(YTimer *timer);
     virtual int handleError(XErrorEvent *xev);
     void runRestart(const char *path, char *const *args);
 
-    FocusModels focusMode;
+    FocusModel focusMode;
     Window managerWindow;
+    ref<YIcon> defaultAppIcon;
 
-    static void initPointers();
-    static void initIcons();
-    static void termIcons();
-    static void initIconSize();
-    static void initPixmaps();
+    MStringArray resourcePaths;
+    YArray<bool> themeOnlyPath;
+
+    void freePointers();
+    void initPointers();
+    void initIcons();
+    void initIconSize();
 };
 
 extern YWMApp * wmapp;
 
-#ifdef __WMPROG_H
+#ifdef WMPROG_H
 class RootMenu  : public StartMenu {
 public:
     RootMenu() : StartMenu(wmapp, wmapp, wmapp, "menu") {
@@ -157,7 +164,7 @@ public:
 extern lazily<RootMenu> rootMenu;
 #endif
 
-#ifdef __WMWINMENU_H
+#ifdef WMWINMENU_H
 class SharedWindowList  : public WindowListMenu {
 public:
     SharedWindowList() : WindowListMenu(wmapp) {
@@ -167,33 +174,35 @@ public:
 extern lazily<SharedWindowList> windowListMenu;
 #endif
 
-class LogoutMenu : public YMenu {
+class SharedMenu : public YMenu {
+public:
+    SharedMenu() { setShared(true); }
+};
+
+class LogoutMenu : public SharedMenu {
 public:
-    LogoutMenu() {
-        setShared(true);
-    }
     void updatePopup();
 };
 extern lazy<LogoutMenu> logoutMenu;
 
-class LayerMenu : public YMenu {
+class LayerMenu : public SharedMenu {
 public:
-    LayerMenu() {
-        setShared(true);
-    }
     void updatePopup();
 };
 extern lazy<LayerMenu> layerMenu;
 
-class MoveMenu : public YMenu {
+class MoveMenu : public SharedMenu {
 public:
-    MoveMenu() {
-        setShared(true);
-    }
     void updatePopup();
 };
 extern lazy<MoveMenu> moveMenu;
 
+class TileMenu : public SharedMenu {
+public:
+    void updatePopup();
+};
+extern lazy<TileMenu> tileMenu;
+
 class KProgram;
 typedef YObjectArray<KProgram> KProgramArrayType;
 typedef KProgramArrayType::IterType KProgramIterType;
