@@ -25,7 +25,7 @@ YFrameButton::YFrameButton(YFrameTitleBar* parent, char kind) :
     fKind(kind),
     fVisible(false)
 {
-    if (onRight())
+    if (parent->isRight(kind))
         setWinGravity(NorthEastGravity);
 
     addStyle(wsNoExpose);
@@ -50,10 +50,6 @@ bool YFrameButton::focused() const {
     return getFrame()->focused();
 }
 
-bool YFrameButton::onRight() const {
-    return (strchr(titleButtonsRight, fKind) != nullptr);
-}
-
 void YFrameButton::setKind(char kind) {
     const unsigned height = wsTitleBar;
     unsigned width;
@@ -163,7 +159,7 @@ void YFrameButton::handleClick(const XButtonEvent &up, int count) {
     }
 }
 
-void YFrameButton::handleBeginDrag(const XButtonEvent &down, const XMotionEvent &/*motion*/) {
+bool YFrameButton::handleBeginDrag(const XButtonEvent &down, const XMotionEvent &/*motion*/) {
     bool dragButton = inrange(int(down.button), Button2, Button3);
     if (dragButton && getFrame()->canMove()) {
         if (!isPopupActive()) {
@@ -172,8 +168,10 @@ void YFrameButton::handleBeginDrag(const XButtonEvent &down, const XMotionEvent
                                       0, 0,
                                       down.x + x() + (tbar ? tbar->x() : 0),
                                       down.y + y() + (tbar ? tbar->y() : 0));
+            return true;
         }
     }
+    return false;
 }
 
 void YFrameButton::handleVisibility(const XVisibilityEvent& visib) {
