@@ -6,29 +6,28 @@
  * Dialogs
  */
 #include "config.h"
-
-#include "ypaths.h"
 #include "wmdialog.h"
-#include "wmaction.h"
-#include "yactionbutton.h"
 #include "wpixmaps.h"
 #include "prefs.h"
 #include "wmapp.h"
 #include "wmmgr.h"
-#include "yrect.h"
 #include "ypointer.h"
-#include "sysdep.h"
-
 #include "intl.h"
 
+#define HORZ 10
+#define MIDH 10
+#define VERT 10
+#define MIDV 6
+
+static YColorName cadBg(&clrDialog);
+
 bool couldRunCommand(const char *cmd) {
     if (isEmpty(cmd))
         return false;
     // else-case. Defined, but check whether it's executable first
     csmart copy(newstr(cmd));
-    char *save = 0;
-    char *tokn = strtok_r(copy, " \t\n", &save);
-    return tokn && findPath(getenv("PATH"), X_OK, tokn) != null;
+    csmart path(path_lookup(tokens(copy, " \t\n")));
+    return path;
 }
 
 bool canLock()
@@ -38,9 +37,9 @@ bool canLock()
 
 bool canShutdown(RebootShutdown reboot) {
     if (reboot == Shutdown && isEmpty(shutdownCommand))
-            return false;
+        return false;
     if (reboot == Reboot && isEmpty(rebootCommand))
-            return false;
+        return false;
     if (nonempty(logoutCommand))
         return false;
 #ifdef CONFIG_SESSION
@@ -50,19 +49,12 @@ bool canShutdown(RebootShutdown reboot) {
     return true;
 }
 
-#define HORZ 10
-#define MIDH 10
-#define VERT 10
-#define MIDV 6
-
-static YColorName cadBg(&clrDialog);
-
 CtrlAltDelete::CtrlAltDelete(IApp *app, YWindow *parent): YWindow(parent) {
     this->app = app;
-    unsigned w = 0, h = 0;
+    unsigned w = 140, h = 22;
 
     setStyle(wsOverrideRedirect);
-    setPointer(YXApplication::leftPointer);
+    setPointer(YWMApp::leftPointer);
     setToplevel(true);
 
     /* Create the following buttons in an ordered sequence,
@@ -95,7 +87,7 @@ CtrlAltDelete::CtrlAltDelete(IApp *app, YWindow *parent): YWindow(parent) {
 
     int dx, dy;
     unsigned dw, dh;
-    manager->getScreenGeometry(&dx, &dy, &dw, &dh);
+    desktop->getScreenGeometry(&dx, &dy, &dw, &dh);
     setPosition(dx + (dw - width()) / 2,
                 dy + (dh - height()) / 2);
 
@@ -115,15 +107,15 @@ CtrlAltDelete::CtrlAltDelete(IApp *app, YWindow *parent): YWindow(parent) {
 }
 
 CtrlAltDelete::~CtrlAltDelete() {
-    delete lockButton; lockButton = 0;
-    delete suspendButton; suspendButton = 0;
-    delete cancelButton; cancelButton = 0;
-    delete logoutButton; logoutButton = 0;
-    delete rebootButton; rebootButton = 0;
-    delete shutdownButton; shutdownButton = 0;
-    delete windowListButton; windowListButton = 0;
-    delete restartButton; restartButton = 0;
-    delete aboutButton; aboutButton = 0;
+    delete lockButton; lockButton = nullptr;
+    delete suspendButton; suspendButton = nullptr;
+    delete cancelButton; cancelButton = nullptr;
+    delete logoutButton; logoutButton = nullptr;
+    delete rebootButton; rebootButton = nullptr;
+    delete shutdownButton; shutdownButton = nullptr;
+    delete windowListButton; windowListButton = nullptr;
+    delete restartButton; restartButton = nullptr;
+    delete aboutButton; aboutButton = nullptr;
 }
 
 void CtrlAltDelete::configure(const YRect2& r) {
@@ -227,17 +219,24 @@ void CtrlAltDelete::deactivate() {
     //manager->setFocus(manager->getFocus());
 }
 
-YActionButton* CtrlAltDelete::addButton(const ustring& str, unsigned& maxW, unsigned& maxH)
+YActionButton* CtrlAltDelete::addButton(const mstring& str, unsigned& maxW, unsigned& maxH)
 {
-    YActionButton* b = new YActionButton(this);
-    b->setText(str, -2);
-    if (b->width() > maxW) maxW = b->width();
-    if (b->height() > maxH) maxH = b->height();
-    b->setActionListener(this);
-    b->show();
+    YActionButton* b = new YActionButton(this, str, -2, this);
+    maxW = max(maxW, b->width());
+    maxH = max(maxH, b->height());
     return b;
 }
 
+YActionButton::YActionButton(YWindow* parent, const mstring& text, int hotkey,
+                             YActionListener* listener):
+    YButton(parent, YAction())
+{
+    addStyle(wsNoExpose);
+    setText(text, hotkey);
+    setActionListener(listener);
+    show();
+}
+
 void YActionButton::repaint() {
     GraphicsBuffer(this).paint();
 }
@@ -248,5 +247,11 @@ void YActionButton::configure(const YRect2& r) {
     }
 }
 
+YDimension YActionButton::getTextSize() {
+    YFont font(getActiveFont());
+    return YDimension(
+            max(72, font ? font->textWidth(getText()) + 12 : 0),
+            max(18, font ? font->height() : 0));
+}
 
 // vim: set sw=4 ts=4 et:
