@@ -6,12 +6,15 @@
 #include "config.h"
 #include "wmprog.h"
 #include "yconfig.h"
+#include "ypointer.h"
 #include "wmapp.h"
 #include "sysdep.h"
 #include "ascii.h"
 #include "argument.h"
 #include "intl.h"
 
+#define TIMEOUT_MS 1400
+
 static char* getWord(char* word, size_t wordsize, char* start) {
     char *p = start;
     while (ASCII::isAlnum(*p))
@@ -30,7 +33,7 @@ static char *getCommandArgs(char *source, Argument *command,
 {
     char *p = source;
     p = YConfig::getArgument(command, p);
-    if (p == 0) {
+    if (p == nullptr) {
         msg(_("Missing command argument"));
         return p;
     }
@@ -48,7 +51,7 @@ static char *getCommandArgs(char *source, Argument *command,
 
         // parse the argument into argx and set the new position
         p = YConfig::getArgument(&argx, p);
-        if (p == 0) {
+        if (p == nullptr) {
             msg(_("Bad argument %d to command '%s'"),
                     args.getCount() + 1, command->cstr());
             return p;
@@ -57,35 +60,29 @@ static char *getCommandArgs(char *source, Argument *command,
         args.append(argx);
         MSG(("ARG: %s\n", argx.cstr()));
     }
-    args.append(0);
+    args.append(nullptr);
 
     return p;
 }
 
-static ustring guessIconNameFromExe(const char* exe)
+static mstring guessIconNameFromExe(const char* exe)
 {
-    upath fullname(exe);
-    char buf[1024];
-    for (int i=7; i; --i)
-    {
-        fullname = findPath(getenv("PATH"), X_OK, fullname);
-        if (fullname == null)
-            return "-";
-        ssize_t linkLen = readlink(fullname.string().c_str(), buf, ACOUNT(buf));
+    csmart path(path_lookup(exe));
+    for (int i = 7; i && path; --i) {
+        char buf[PATH_MAX];
+        ssize_t linkLen = readlink(path, buf, PATH_MAX);
         if (linkLen < 0)
             break;
-        fullname = upath(buf, linkLen);
+        path = newstr(buf, linkLen);
     }
-    // crop to the generic name
-    ustring s(fullname);
-    int spos = s.lastIndexOf('/');
-    if (spos >= 0)
-        s = s.remove(0, spos + 1);
-    // scripts have a suffix sometimes which is not part of the icon name
-    spos = s.indexOf('.');
-    if (spos >= 0)
-        s = s.substring(0, spos);
-    return s;
+    if (path) {
+        char* base = const_cast<char*>(my_basename(path));
+        // scripts may have a suffix which is not part of the icon name
+        char* dot = strchr(base, '.');
+        if (dot) *dot = '\0';
+        return base;
+    }
+    return "-";
 }
 
 char* MenuLoader::parseKey(char *word, char *p)
@@ -96,20 +93,20 @@ char* MenuLoader::parseKey(char *word, char *p)
     Argument key;
 
     p = YConfig::getArgument(&key, p);
-    if (p == 0) return p;
+    if (p == nullptr) return p;
 
     Argument wmclass;
 
     if (runonce) {
         p = YConfig::getArgument(&wmclass, p);
-        if (p == 0) return p;
+        if (p == nullptr) return p;
     }
 
     Argument command;
     YStringArray args;
 
     p = getCommandArgs(p, &command, args);
-    if (p == 0) {
+    if (p == nullptr) {
         msg(_("Error at keyword '%s' for %s"), word, key.cstr());
         return p;
     }
@@ -120,7 +117,7 @@ char* MenuLoader::parseKey(char *word, char *p)
         key,
         null,
         false,
-        runonce ? wmclass.cstr() : NULL,
+        runonce ? wmclass.cstr() : nullptr,
         command.cstr(),
         args);
 
@@ -137,25 +134,25 @@ char* MenuLoader::parseProgram(char *word, char *p, ObjectContainer *container)
     Argument name;
 
     p = YConfig::getArgument(&name, p);
-    if (p == 0) return p;
+    if (p == nullptr) return p;
 
     Argument icons;
 
     p = YConfig::getArgument(&icons, p);
-    if (p == 0) return p;
+    if (p == nullptr) return p;
 
     Argument wmclass;
 
     if (runonce) {
         p = YConfig::getArgument(&wmclass, p);
-        if (p == 0) return p;
+        if (p == nullptr) return p;
     }
 
     Argument command;
     YStringArray args;
 
     p = getCommandArgs(p, &command, args);
-    if (p == 0) {
+    if (p == nullptr) {
         msg(_("Error at %s '%s'"), word, name.cstr());
         return p;
     }
@@ -164,9 +161,9 @@ char* MenuLoader::parseProgram(char *word, char *p, ObjectContainer *container)
 
     if (icons[0] == '!')
     {
-        ustring iconName = guessIconNameFromExe(command);
+        mstring iconName = guessIconNameFromExe(command);
         if (iconName.charAt(0) != '-')
-            icon = YIcon::getIcon(cstring(iconName));
+            icon = YIcon::getIcon(iconName);
 
     }
     else if (icons[0] != '-')
@@ -178,7 +175,7 @@ char* MenuLoader::parseProgram(char *word, char *p, ObjectContainer *container)
         name,
         icon,
         restart,
-        runonce ? wmclass.cstr() : NULL,
+        runonce ? wmclass.cstr() : nullptr,
         command.cstr(),
         args);
 
@@ -192,19 +189,19 @@ char* MenuLoader::parseAMenu(char *p, ObjectContainer *container)
     Argument name;
 
     p = YConfig::getArgument(&name, p);
-    if (p == 0) return p;
+    if (p == nullptr) return p;
 
     Argument icons;
 
     p = YConfig::getArgument(&icons, p);
-    if (p == 0) return p;
+    if (p == nullptr) return p;
 
     while (ASCII::isWhiteSpace(*p))
         ++p;
 
     char word[32];
     p = getWord(word, sizeof(word), p);
-    if (*p != '{') return 0;
+    if (*p != '{') return nullptr;
     p++;
 
     ref<YIcon> icon;
@@ -233,17 +230,17 @@ char* MenuLoader::parseMenuFile(char *p, ObjectContainer *container)
     Argument name;
 
     p = YConfig::getArgument(&name, p);
-    if (p == 0) return p;
+    if (p == nullptr) return p;
 
     Argument icons;
 
     p = YConfig::getArgument(&icons, p);
-    if (p == 0) return p;
+    if (p == nullptr) return p;
 
     Argument menufile;
 
     p = YConfig::getArgument(&menufile, p);
-    if (p == 0) return p;
+    if (p == nullptr) return p;
 
     ref<YIcon> icon;
     if (icons[0] != '-')
@@ -251,7 +248,7 @@ char* MenuLoader::parseMenuFile(char *p, ObjectContainer *container)
 
     ObjectMenu *filemenu = new MenuFileMenu(
             app, smActionListener, wmActionListener,
-            menufile.cstr(), 0);
+            menufile.cstr(), nullptr);
 
     if (menufile)
         container->addContainer(name.cstr(), icon, filemenu);
@@ -264,18 +261,18 @@ char* MenuLoader::parseMenuProg(char *p, ObjectContainer *container)
     Argument name;
 
     p = YConfig::getArgument(&name, p);
-    if (p == 0) return p;
+    if (p == nullptr) return p;
 
     Argument icons;
 
     p = YConfig::getArgument(&icons, p);
-    if (p == 0) return p;
+    if (p == nullptr) return p;
 
     Argument command;
     YStringArray args;
 
     p = getCommandArgs(p, &command, args);
-    if (p == 0) {
+    if (p == nullptr) {
         msg(_("Error at menuprog '%s'"), name.cstr());
         return p;
     }
@@ -285,8 +282,8 @@ char* MenuLoader::parseMenuProg(char *p, ObjectContainer *container)
         icon = YIcon::getIcon(icons);
     MSG(("menuprog %s %s", name.cstr(), command.cstr()));
 
-    upath fullPath = findPath(getenv("PATH"), X_OK, command.cstr());
-    if (fullPath != null) {
+    csmart path(path_lookup(command.cstr()));
+    if (path) {
         ObjectMenu *progmenu = new MenuProgMenu(
                 app, smActionListener, wmActionListener,
                 name.cstr(), command.cstr(), args);
@@ -302,24 +299,24 @@ char* MenuLoader::parseMenuProgReload(char *p, ObjectContainer *container)
     Argument name;
 
     p = YConfig::getArgument(&name, p);
-    if (p == 0) return p;
+    if (p == nullptr) return p;
 
     Argument icons;
 
     p = YConfig::getArgument(&icons, p);
-    if (p == 0) return p;
+    if (p == nullptr) return p;
 
     Argument timeoutStr;
 
     p = YConfig::getArgument(&timeoutStr, p);
-    if (p == 0) return p;
+    if (p == nullptr) return p;
     long timeout = atol(timeoutStr);
 
     Argument command;
     YStringArray args;
 
     p = getCommandArgs(p, &command, args);
-    if (p == 0) {
+    if (p == nullptr) {
         msg(_("Error at menuprogreload: '%s'"), name.cstr());
         return p;
     }
@@ -329,8 +326,8 @@ char* MenuLoader::parseMenuProgReload(char *p, ObjectContainer *container)
         icon = YIcon::getIcon(icons);
     MSG(("menuprogreload %s %s", name.cstr(), command.cstr()));
 
-    upath fullPath = findPath(getenv("PATH"), X_OK, command.cstr());
-    if (fullPath != null) {
+    csmart path(path_lookup(command.cstr()));
+    if (path) {
         ObjectMenu *progmenu = new MenuProgMenu(
                 app, smActionListener, wmActionListener,
                 name.cstr(), command.cstr(), args, timeout);
@@ -346,7 +343,7 @@ char* MenuLoader::parseIncludeStatement(char *p, ObjectContainer *container)
     Argument filename;
 
     p = YConfig::getArgument(&filename, p);
-    if (p == 0) {
+    if (p == nullptr) {
         warn(_("Missing filename argument to include statement"));
         return p;
     }
@@ -364,7 +361,7 @@ char* MenuLoader::parseIncludeProgStatement(char *p, ObjectContainer *container)
     YStringArray args;
 
     p = getCommandArgs(p, &command, args);
-    if (p == 0) {
+    if (p == nullptr) {
         msg(_("Error at includeprog '%s'"), command.cstr());
         return p;
     }
@@ -409,7 +406,7 @@ char* MenuLoader::parseWord(char *word, char *p, ObjectContainer *container)
         }
         else {
             msg(_("Unknown keyword '%s'"), word);
-            return 0;
+            return nullptr;
         }
     }
     else if (!strcmp(word, "key")
@@ -421,7 +418,7 @@ char* MenuLoader::parseWord(char *word, char *p, ObjectContainer *container)
     else {
         msg(_("Unknown keyword for a non-container: '%s'.\n"
               "Expected either 'key' or 'runonce' here.\n"), word);
-        return 0;
+        return nullptr;
     }
     return p;
 }
@@ -444,7 +441,7 @@ char* MenuLoader::parseMenus(char *data, ObjectContainer *container)
             p = parseWord(word, p, container);
         }
     }
-    return 0;
+    return nullptr;
 }
 
 void MenuLoader::loadMenus(upath menufile, ObjectContainer *container)
@@ -452,13 +449,10 @@ void MenuLoader::loadMenus(upath menufile, ObjectContainer *container)
     if (menufile.isEmpty())
         return;
 
-    MSG(("menufile: %s", menufile.string().c_str()));
+    MSG(("menufile: %s", menufile.string()));
     YTraceConfig trace(menufile.string());
-    char *buf = menufile.loadText();
-    if (buf) {
-        parseMenus(buf, container);
-        delete[] buf;
-    }
+    auto buf = menufile.loadText();
+    if (buf) parseMenus(buf, container);
 }
 
 void MenuLoader::progMenus(
@@ -466,48 +460,54 @@ void MenuLoader::progMenus(
     char *const argv[],
     ObjectContainer *container)
 {
-    fileptr fpt(tmpfile());
-    if (fpt == 0) {
-        fail("tmpfile");
+    int fds[2];
+    if (pipe(fds) == -1) {
+        fail("pipe");
         return;
     }
 
-    int tfd = fileno(fpt);
-    int status = 0;
-    pid_t child_pid = fork();
-
-    if (child_pid == -1) {
-        fail("Forking '%s' failed", command);
+    int pid = fork();
+    if (pid == -1) {
+        fail(_("Failed to execute %s"), command);
     }
-    else if (child_pid == 0) {
+    else if (pid == 0) {
+        close(fds[0]);
         int devnull = open("/dev/null", O_RDONLY);
         if (devnull > 0) {
             dup2(devnull, 0);
             close(devnull);
         }
-        if (dup2(tfd, 1) == 1) {
-            if (tfd > 2) close(tfd);
-            execvp(command, argv);
-        }
-        fail("Exec '%s' failed", command);
-        _exit(99);
-    }
-    else if (waitpid(child_pid, &status, 0) == 0 && status != 0) {
-        warn("'%s' exited with code %d.", command, status);
-    }
-    else if (lseek(tfd, (off_t) 0L, SEEK_SET) == (off_t) -1) {
-        fail("lseek failed");
-    }
-    else {
-        char *buf = load_fd(tfd);
-        fpt.close();
-        if (buf && *buf) {
-            parseMenus(buf, container);
+        if (fds[1] > 1 && (dup2(fds[1], 1) != 1 || close(fds[1]))) {
+            fail("dup2!=1");
         }
         else {
+            char* path = path_lookup(command);
+            if (path) {
+                execv(path, argv);
+            }
+            fail(_("Failed to execute %s"), path ? path : command);
+        }
+        _exit(99);
+    }
+    else {
+        close(fds[1]);
+        bool expired = false;
+        filereader rdr(fds[0]);
+        auto buf = rdr.read_pipe(TIMEOUT_MS, &expired);
+        if (expired) {
+            warn(_("'%s' timed out!"), command);
+            kill(pid, SIGKILL);
+        }
+        int status = app->waitProgram(pid);
+        if (status) {
+            tlog(_("%s exited with status %d."), command, status);
+        }
+        else if (nonempty(buf)) {
+            parseMenus(buf, container);
+        }
+        else if (expired == false) {
             warn(_("'%s' produces no output"), command);
         }
-        delete[] buf;
     }
 }
 
