@@ -3,8 +3,8 @@
 
 #include "WinMgr.h"
 #include "ylist.h"
-#include "yaction.h"
 #include "ymsgbox.h"
+#include "ypopup.h"
 #include "workspaces.h"
 
 extern YAction layerActionSet[WinLayerCount];
@@ -14,19 +14,24 @@ class YWindowManager;
 class YFrameClient;
 class YFrameWindow;
 class YSMListener;
+class SwitchWindow;
+class MiniIcon;
+class DockApp;
 class IApp;
 
-class EdgeSwitch: public YWindow, public YTimerListener {
+class EdgeSwitch: public YDndWindow, public YTimerListener {
 public:
     EdgeSwitch(YWindowManager *manager, int delta, bool vertical);
     virtual ~EdgeSwitch();
 
     virtual void handleCrossing(const XCrossingEvent &crossing);
     virtual bool handleTimer(YTimer *t);
+    virtual void handleDNDEnter();
+    virtual void handleDNDLeave();
     void setGeometry();
 private:
     YWindowManager *fManager;
-    YCursor & fCursor;
+    Cursor fCursor;
     int fDelta;
     bool fVert;
 
@@ -41,7 +46,37 @@ public:
     virtual void handleButton(const XButtonEvent &button);
 };
 
-class YWindowManager: private YDesktop, private YMsgBoxListener {
+class YArrange {
+public:
+    YArrange(YFrameWindow** w, int n) : win(w), count(n) { }
+    void discard() { delete[] win; win = nullptr; count = 0; }
+    int size() const { return count; }
+    YFrameWindow** begin() const { return win; }
+    YFrameWindow** end() const { return win + count; }
+    YFrameWindow* operator[](int index) const { return win[index]; }
+    operator bool() const { return win && count; }
+private:
+    YFrameWindow** win;
+    int count;
+};
+
+class YTopWindow : public YWindow {
+public:
+    YTopWindow();
+    bool handleKey(const XKeyEvent& key) override;
+    void setFrame(YFrameWindow* frame);
+
+private:
+    YFrameWindow* fFrame;
+    Window fHandle;
+};
+
+class YWindowManager:
+    private YDesktop,
+    private YMsgBoxListener,
+    private YTimerListener,
+    public YPopDownListener
+{
 public:
     YWindowManager(
         IApp *app,
@@ -70,6 +105,8 @@ public:
     virtual void handleRRNotify(const XRRNotifyEvent &notify);
 #endif
     virtual void handleMsgBox(YMsgBox *msgbox, int operation);
+    virtual void handlePopDown(YPopupWindow* popup);
+    virtual bool handleTimer(YTimer* timer);
 
     void manageClients();
     void unmanageClients();
@@ -82,12 +119,12 @@ public:
 
     YFrameWindow *findFrame(Window win);
     YFrameClient *findClient(Window win);
-    void manageClient(Window win, bool mapClient = false);
+    void manageClient(YFrameClient* client, bool mapClient = false);
     void unmanageClient(YFrameClient *client);
     void destroyedClient(Window win);
     void mapClient(Window win);
 
-    void setFocus(YFrameWindow *f, bool canWarp = false);
+    void setFocus(YFrameWindow *f, bool canWarp = false, bool reorder = true);
     YFrameWindow *getFocus() { return fFocusWin; }
 
     void installColormap(Colormap cmap);
@@ -97,12 +134,13 @@ public:
     void removeClientFrame(YFrameWindow *frame);
 
     void updateScreenSize(XEvent *event);
+    void getWorkArea(int *mx, int *my, int *Mx, int *My);
     void getWorkArea(const YFrameWindow *frame, int *mx, int *my, int *Mx, int *My, int xiscreen = -1);
     void getWorkAreaSize(YFrameWindow *frame, int *Mw,int *Mh);
 
     int calcCoverage(bool down, YFrameWindow *frame, int x, int y, int w, int h);
     void tryCover(bool down, YFrameWindow *frame, int x, int y, int w, int h,
-                  int &px, int &py, int &cover, int xiscreen);
+                  int& px, int& py, int& cover, int mx, int my, int Mx, int My);
     bool getSmartPlace(bool down, YFrameWindow *frame, int &x, int &y, int w, int h, int xiscreen);
     void getNewPosition(YFrameWindow *frame, int &x, int &y, int w, int h, int xiscreen);
     void placeWindow(YFrameWindow *frame, int x, int y, int cw, int ch, bool newClient, bool &canActivate);
@@ -111,6 +149,7 @@ public:
     void setTop(long layer, YFrameWindow *top);
     YFrameWindow *bottom(long layer) const;
     void setBottom(long layer, YFrameWindow *bottom);
+    YWindow* bottomWindow() const { return fBottom; }
 
     YFrameWindow *topLayer(long layer = WinLayerCount - 1);
     YFrameWindow *bottomLayer(long layer = 0);
@@ -138,14 +177,13 @@ public:
     void updateClientList();
     void updateUserTime(const UserTime& userTime);
 
-    YMenu *createWindowMenu(YMenu *menu, long workspace);
     int windowCount(long workspace);
     void popupWindowListMenu(YWindow *owner, int x, int y);
 
     void initWorkspaces();
 
-    long activeWorkspace() const { return fActiveWorkspace; }
-    long lastWorkspace() const { return fLastWorkspace; }
+    int activeWorkspace() const { return int(fActiveWorkspace); }
+    int lastWorkspace() const { return int(fLastWorkspace); }
     void activateWorkspace(long workspace);
 
     void appendNewWorkspaces(long extra);
@@ -180,7 +218,7 @@ public:
     void workAreaUpdated();
     void resizeWindows();
 
-    void getIconPosition(YFrameWindow *frame, int *iconX, int *iconY);
+    void getIconPosition(MiniIcon* iw, int *iconX, int *iconY);
 
     void wmCloseSession();
     void exitAfterLastClient(bool shuttingDown);
@@ -201,17 +239,22 @@ public:
     void switchToLastWorkspace(bool takeCurrent);
 
     void tilePlace(YFrameWindow *w, int tx, int ty, int tw, int th);
-    void tileWindows(YFrameWindow **w, int count, bool vertical);
-    void smartPlace(YFrameWindow **w, int count);
+    void tileWindows(YArrange arrange, bool vertical);
+    void smartPlace(YArrange arrange);
     void getCascadePlace(YFrameWindow *frame, int &lastX, int &lastY, int &x, int &y, int w, int h);
-    void cascadePlace(YFrameWindow **w, int count);
-    void setWindows(YFrameWindow **w, int count, YAction action);
+    void cascadePlace(YArrange arrange);
+    void setWindows(YArrange arrange, YAction action);
 
-    bool getWindowsToArrange(YFrameWindow ***w, int *count, bool sticky = false, bool skipNonMinimizable = false);
+    YArrange getWindowsToArrange(bool sticky = false, bool skipNonMinimizable = false);
 
-    void saveArrange(YFrameWindow **w, int count);
+    bool saveArrange(YArrange arrange);
     void undoArrange();
     void arrangeIcons();
+    void tileWindows(bool vertical);
+    void arrangeWindows();
+    void actionWindows(YAction action);
+    void toggleDesktop();
+    void cascadeWindows();
 
     bool haveClients();
     void setupRootProxy();
@@ -219,12 +262,10 @@ public:
     void setKeyboard(int configIndex);
     mstring getKeyboard();
     void updateKeyboard(int configIndex);
+    void reflectKeyboard(int configIndex, mstring keyboard);
+    void kbLayout();
 
-    void updateFullscreenLayer();
-    void updateFullscreenLayerEnable(bool enable);
-    int getSwitchScreen();
-
-    static void doWMAction(WMAction action);
+    static void doWMAction(WMAction action, bool putback = false);
     void lockFocus() {
         //MSG(("lockFocus %d", lockFocusCount));
         lockFocusCount++;
@@ -241,9 +282,20 @@ public:
     }
     void requestWorkAreaUpdate() { ++fWorkAreaUpdate; }
 
+    void lockRestack() { fRestackLock++; }
+    void unlockRestack() {
+        if (0 == --fRestackLock && fRestackUpdate) {
+            fRestackUpdate = 0;
+            restackWindows();
+        }
+    }
+
     enum WMState { wmSTARTUP, wmRUNNING, wmSHUTDOWN };
 
     WMState wmState() const { return fWmState; }
+    bool isStartup() const { return fWmState == wmSTARTUP; }
+    bool isRunning() const { return fWmState == wmRUNNING; }
+    bool notRunning() const { return fWmState != wmRUNNING; }
     bool fullscreenEnabled() { return fFullscreenEnabled; }
     void setFullscreenEnabled(bool enable) { fFullscreenEnabled = enable; }
     const UserTime& lastUserTime() const { return fLastUserTime; }
@@ -258,6 +310,12 @@ public:
     };
 
     const DesktopLayout& layout() const { return fLayout; }
+    bool handleSwitchWorkspaceKey(const XKeyEvent& key, KeySym k, unsigned vm);
+
+    int getSwitchScreen();
+    bool switchWindowVisible() const;
+    SwitchWindow* getSwitchWindow();
+    Window netActiveWindow() const { return fActiveWindow; }
 
 private:
     struct WindowPosState {
@@ -266,8 +324,10 @@ private:
         YFrameWindow *frame;
     };
 
+    YFrameClient* allocateClient(Window win, bool mapClient);
+    YFrameWindow* allocateFrame(YFrameClient* client);
     void updateArea(long workspace, int screen_number, int l, int t, int r, int b);
-    bool handleWMKey(const XKeyEvent &key, KeySym k, unsigned int m, unsigned int vm);
+    bool handleWMKey(const XKeyEvent &key, KeySym k, unsigned vm);
     void setWmState(WMState newWmState);
     void refresh();
 
@@ -314,12 +374,17 @@ private:
     int fArrangeCount;
     WindowPosState *fArrangeInfo;
     YProxyWindow *rootProxy;
-    YWindow *fTopWin;
+    YTopWindow *fTopWin;
+    YWindow *fBottom;
+    int fCascadeX;
+    int fCascadeY;
     int fIconColumn;
     int fIconRow;
     int lockFocusCount;
     int fWorkAreaLock;
     int fWorkAreaUpdate;
+    int fRestackLock;
+    int fRestackUpdate;
     int fServerGrabCount;
     bool fFullscreenEnabled;
 
@@ -332,10 +397,26 @@ private:
     DesktopLayout fLayout;
     mstring fCurrentKeyboard;
     int fDefaultKeyboard;
+    SwitchWindow* fSwitchWindow;
+    lazy<YTimer> fSwitchDownTimer;
+    lazy<YTimer> fLayoutTimer;
+    DockApp* fDockApp;
 };
 
 extern YWindowManager *manager;
 
+class YRestackLock {
+public:
+    YRestackLock() { manager->lockRestack(); }
+    ~YRestackLock() { manager->unlockRestack(); }
+};
+
+class YFullscreenLock {
+public:
+    YFullscreenLock() { manager->setFullscreenEnabled(false); }
+    ~YFullscreenLock() { manager->setFullscreenEnabled(true); }
+};
+
 void dumpZorder(const char *oper, YFrameWindow *w, YFrameWindow *a = nullptr);
 
 extern Atom _XA_WIN_ICONS;
