@@ -10,13 +10,9 @@
 #include "yxapp.h"
 #include "prefs.h"
 
-static ref<YFont> minimizedWindowFont;
-static YColorName normalMinimizedWindowBg(&clrNormalMinimizedWindow);
-static YColorName normalMinimizedWindowFg(&clrNormalMinimizedWindowText);
-static YColorName activeMinimizedWindowBg(&clrActiveMinimizedWindow);
-static YColorName activeMinimizedWindowFg(&clrActiveMinimizedWindowText);
+MiniIcon::ArrayType MiniIcon::fIcons;
 
-static bool acceptableDimensions(unsigned w, unsigned h) {
+bool MiniIcon::acceptableDimensions(unsigned w, unsigned h) const {
     unsigned lower = YIcon::hugeSize() / 2;
     unsigned upper = 3 * YIcon::hugeSize() / 2;
     return inrange(w, lower, upper) && inrange(h, lower, upper);
@@ -37,14 +33,13 @@ MiniIcon::MiniIcon(YFrameWindow *frame):
 {
     setStyle(wsOverrideRedirect | wsBackingMapped);
     setSize(YIcon::hugeSize(), YIcon::hugeSize());
+    setNetWindowType(_XA_NET_WM_WINDOW_TYPE_DESKTOP);
     setTitle("MiniIcon");
     setPosition(-1, -1);
 
-    if (minimizedWindowFont == null)
-        minimizedWindowFont = YFont::getFont(XFA(minimizedWindowFontName));
-
     if (fIconWindow) {
-        Window root, parent, *child;
+        Window root, parent;
+        xsmart<Window> child;
         unsigned border, depth, count;
         if (XGetGeometry(xapp->display(), fIconWindow, &root,
                          &fIconGeometry.xx, &fIconGeometry.yy,
@@ -84,6 +79,8 @@ MiniIcon::MiniIcon(YFrameWindow *frame):
     }
 
     updateIcon();
+
+    fIcons += this;
 }
 
 MiniIcon::~MiniIcon() {
@@ -93,6 +90,7 @@ MiniIcon::~MiniIcon() {
                         fIconGeometry.xx, fIconGeometry.hh);
         XRemoveFromSaveSet(xapp->display(), fIconWindow);
     }
+    findRemove(fIcons, this);
 }
 
 void MiniIcon::handleExpose(const XExposeEvent& expose) {
@@ -146,16 +144,20 @@ void MiniIcon::updateIcon() {
 void MiniIcon::updatePosition() {
     int xpos = x();
     int ypos = y();
-    manager->getIconPosition(fFrame, &xpos, &ypos);
+    manager->getIconPosition(this, &xpos, &ypos);
     if (xpos != x() || ypos != y()) {
         setPosition(xpos, ypos);
     }
 }
 
+void MiniIcon::stack() {
+    beneath(manager->bottomWindow());
+}
+
 void MiniIcon::show() {
     updatePosition();
     if (x() != -1 || y() != -1) {
-        beneath(manager->bottomLayer());
+        stack();
         YWindow::show();
     }
 }
@@ -183,7 +185,7 @@ void MiniIcon::handleClick(const XButtonEvent &up, int /*count*/) {
     }
     else if (up.button == Button1) {
         if (up.state & xapp->AltMask) {
-            lower();
+            stack();
         } else {
             if (!(up.state & ControlMask))
                 getFrame()->wmRaise();
@@ -202,13 +204,14 @@ void MiniIcon::handleCrossing(const XCrossingEvent &crossing) {
     YWindow::handleCrossing(crossing);
 }
 
-void MiniIcon::handleBeginDrag(const XButtonEvent& d, const XMotionEvent& m) {
+bool MiniIcon::handleBeginDrag(const XButtonEvent& d, const XMotionEvent& m) {
     setToolTip(null);
     raise();
+    return true;
 }
 
 void MiniIcon::handleEndDrag(const XButtonEvent& d, const XButtonEvent& u) {
-    beneath(manager->bottomLayer());
+    stack();
 }
 
 void MiniIcon::handleDrag(const XButtonEvent &down, const XMotionEvent &motion) {
