@@ -14,6 +14,7 @@
 #include "wmclient.h"
 #include "wmmgr.h"
 #include "prefs.h"
+#include "yprefs.h"
 #include "yrect.h"
 #include "workspaces.h"
 #include "intl.h"
@@ -24,7 +25,7 @@
 YColorName YWindowManagerStatus::statusFg(&clrMoveSizeStatusText);
 YColorName YWindowManagerStatus::statusBg(&clrMoveSizeStatus);
 
-ref<YFont> YWindowManagerStatus::statusFont;
+YFont YWindowManagerStatus::statusFont;
 
 lazy<MoveSizeStatus> statusMoveSize;
 lazy<WorkspaceStatus> statusWorkspace;
@@ -34,6 +35,7 @@ lazy<WorkspaceStatus> statusWorkspace;
 
 YWindowManagerStatus::YWindowManagerStatus()
     : YWindow()
+    , fScreen(-1)
     , fConfigured(false)
 {
     setStyle(wsOverrideRedirect | wsSaveUnder | wsNoExpose);
@@ -57,13 +59,12 @@ void YWindowManagerStatus::configureStatus() {
     fConfigured = true;
 
     if (statusFont == null)
-        statusFont = YFont::getFont(XFA(statusFontName));
+        statusFont = statusFontName;
 
     int sW = statusFont->textWidth(longestStatus());
     int sH = statusFont->height();
 
-    int scn = desktop->getScreenForRect(x(), y(), width(), height());
-    YRect geo(desktop->getScreenGeometry(scn));
+    YRect geo(desktop->getScreenGeometry(fScreen));
     setGeometry(YRect(geo.x() + (geo.width() - sW) / 2,
                       geo.y() + (geo.height() - sH) - 8, // / 2,
                       sW + 2, sH + 4));
@@ -88,19 +89,20 @@ void YWindowManagerStatus::paint(Graphics &g, const YRect &/*r*/) {
         else
             g.fillRect(x, y, w, h);
     }
-    g.setColor(statusFg);
-    g.setFont(statusFont);
-
-    mstring status(getStatus());
-    g.drawChars(status,
-                width() / 2 - statusFont->textWidth(status) / 2,
-                height() - statusFont->descent() - 2);
+    if (statusFont) {
+        mstring status(getStatus());
+        g.setFont(statusFont);
+        g.setColor(statusFg);
+        g.drawChars(status,
+                    width() / 2 - statusFont->textWidth(status) / 2,
+                    height() - statusFont->descent() - 2);
+    }
 }
 
 void YWindowManagerStatus::begin() {
+    YRect geo(desktop->getScreenGeometry(fScreen));
     setPosition(x(),
-                 taskBarAtTop ? 4 :
-                 (desktop->height() - height()) - 4);
+                taskBarAtTop ? 4 : int(geo.height() - height()) - 4);
     raise();
     show();
 }
@@ -153,6 +155,7 @@ void MoveSizeStatus::setStatus(YFrameWindow *frame, const YRect &r) {
         fW = width;
         fH = height;
     }
+    setScreen(desktop->getScreenForRect(r.x(), r.y(), r.width(), r.height()));
     repaintSync();
 }
 
@@ -168,6 +171,7 @@ void MoveSizeStatus::setStatus(YFrameWindow *frame) {
         fW = frame->client()->width();
         fH = frame->client()->height();
     }
+    setScreen(frame->getScreen());
     repaintSync();
 }
 
