@@ -17,7 +17,7 @@ class KeyboardStatus;
 class MailBoxControl;
 class MailBoxStatus;
 class YButton;
-class YClock;
+class ClockSet;
 class YApm;
 class TaskBarMenu;
 class TaskPane;
@@ -83,8 +83,6 @@ private:
     virtual void configure(const YRect2 &r);
     virtual void repaint();
 
-    YClock *clock() { return fClock; }
-
 public:
     bool windowTrayRequestDock(Window w);
     void setWorkspaceActive(long workspace, bool active);
@@ -113,10 +111,10 @@ public:
     void relayoutNow();
 
     void detachDesktopTray();
-
+    bool isCollapsed() const { return fIsCollapsed; }
     bool hidden() const { return fIsCollapsed | fIsHidden | !getFrame(); }
     bool autoTimer(bool show);
-    void updateFullscreen(bool fullscreen);
+    void updateFullscreen();
     Window edgeTriggerWindow() { return fEdgeTrigger->handle(); }
     void switchToPrev();
     void switchToNext();
@@ -126,6 +124,7 @@ public:
 
 private:
     void popOut();
+    void obtainFocus();
 
     AddressBar *addressBar() const { return fAddressBar; }
     TaskPane *taskPane() const { return fTasks; }
@@ -148,7 +147,7 @@ private:
 
     ObjectButton *fCollapseButton;
     TrayPane *fWindowTray;
-    YClock *fClock;
+    ClockSet* fClock;
     KeyboardStatus *fKeyboardStatus;
     MailBoxControl *fMailBoxControl;
     MEMStatus *fMEMStatus;
