@@ -14,7 +14,10 @@
 #include "prefs.h"
 #include "intl.h"
 
-static ref<YFont> titleFont;
+#define ISMASK(w,e,n) (((w) & ~(n)) == (e))
+
+static YFont titleFont;
+bool YFrameTitleBar::swapTitleButtons;
 
 static YColorName titleBarBackground[2] = {
     &clrInactiveTitleBar, &clrActiveTitleBar
@@ -28,7 +31,31 @@ static YColorName titleBarShadowText[2] = {
 
 void YFrameTitleBar::initTitleColorsFonts() {
     if (titleFont == null) {
-        titleFont = YFont::getFont(XFA(titleFontName));
+        titleFont = titleFontName;
+        if (rightToLeft) {
+            swapTitleButtons = true;
+            int pn = 0;
+            ref<YPixmap> old;
+            for (char c : YRange<const char>(titleButtonsSupported,
+                                      strlen(titleButtonsSupported))) {
+                ref<YPixmap> pix;
+                switch (c) {
+                    case Depth : pix = depthPixmap[pn]; break;
+                    case Hide  : pix = hidePixmap[pn]; break;
+                    case Mini  : pix = minimizePixmap[pn]; break;
+                    case Maxi  : pix = maximizePixmap[pn]; break;
+                    case Roll  : pix = rollupPixmap[pn]; break;
+                    case Menu  : pix = ::menuButton[pn]; break;
+                    case Close : pix = closePixmap[pn]; break;
+                }
+                if (pix != null) {
+                    if (old != null) {
+                        swapTitleButtons &= (pix->width() == old->width());
+                    }
+                    old = pix;
+                }
+            }
+        }
     }
 }
 
@@ -63,7 +90,12 @@ YFrameTitleBar::~YFrameTitleBar() {
 }
 
 bool YFrameTitleBar::isRight(char c) {
-    return (strchr(titleButtonsRight, c) != nullptr);
+    const char* distant = swapTitleButtons ? titleButtonsLeft : titleButtonsRight;
+    return (strchr(distant, c) != nullptr);
+}
+
+bool YFrameTitleBar::isRight(const YFrameButton* b) {
+    return isRight(b->getKind());
 }
 
 bool YFrameTitleBar::supported(char c) {
@@ -108,7 +140,7 @@ void YFrameTitleBar::handleButton(const XButtonEvent &button) {
     }
     else if (button.type == ButtonRelease) {
         if ((button.button == Button1 || button.button == Button3) &&
-            IS_BUTTON(button.state, Button1Mask + Button3Mask))
+            xapp->isButton(button.state, Button1Mask + Button3Mask))
         {
             windowList->showFocused(button.x_root, button.y_root);
         }
@@ -119,27 +151,29 @@ void YFrameTitleBar::handleButton(const XButtonEvent &button) {
 void YFrameTitleBar::handleClick(const XButtonEvent &up, int count) {
     YAction action(actionNull);
     if (count >= 2 && (count % 2 == 0)) {
-        if (up.button == (unsigned) titleMaximizeButton &&
+        if (up.button == titleMaximizeButton &&
             ISMASK(KEY_MODMASK(up.state), 0, ControlMask))
         {
             action = actionMaximize;
         }
-        else if (up.button == (unsigned) titleMaximizeButton &&
+        else if (up.button == titleMaximizeButton &&
              ISMASK(KEY_MODMASK(up.state), ShiftMask, ControlMask))
         {
             action = actionMaximizeVert;
         }
-        else if (up.button == (unsigned) titleMaximizeButton && xapp->AltMask &&
+        else if (up.button == titleMaximizeButton && xapp->AltMask &&
              ISMASK(KEY_MODMASK(up.state), xapp->AltMask + ShiftMask, ControlMask))
         {
             action = actionMaximizeHoriz;
         }
-        else if (up.button == (unsigned) titleRollupButton &&
+        else if (up.button == titleRollupButton &&
+             up.button <= Button3 &&
              ISMASK(KEY_MODMASK(up.state), 0, ControlMask))
         {
             action = actionRollup;
         }
-        else if (up.button == (unsigned) titleRollupButton &&
+        else if (up.button == titleRollupButton &&
+             up.button <= Button3 &&
              ISMASK(KEY_MODMASK(up.state), ShiftMask, ControlMask))
         {
             action = actionMaximizeHoriz;
@@ -166,13 +200,21 @@ void YFrameTitleBar::handleClick(const XButtonEvent &up, int count) {
                 }
             }
         }
+        else if (inrange<unsigned>(up.button, Button4, Button5) &&
+                 inrange<unsigned>(titleRollupButton, Button4, Button5))
+        {
+            if (up.button == Button4 && !getFrame()->isRollup())
+                action = actionRollup;
+            if (up.button == Button5 && getFrame()->isRollup())
+                action = actionRollup;
+        }
     }
     if (action != actionNull) {
         getFrame()->actionPerformed(action, up.state);
     }
 }
 
-void YFrameTitleBar::handleBeginDrag(
+bool YFrameTitleBar::handleBeginDrag(
         const XButtonEvent &down,
         const XMotionEvent &motion)
 {
@@ -183,13 +225,15 @@ void YFrameTitleBar::handleBeginDrag(
          down.subwindow == getFrame()->topLeftIndicator() ||
          down.subwindow == getFrame()->topRightIndicator()))
     {
-        getFrame()->handleBeginDrag(down, motion);
+        return getFrame()->handleBeginDrag(down, motion);
     }
     else if (getFrame()->canMove()) {
         getFrame()->startMoveSize(true, true,
                                   0, 0,
                                   down.x + x(), down.y + y());
+        return true;
     }
+    return false;
 }
 
 void YFrameTitleBar::activate() {
@@ -211,13 +255,14 @@ void YFrameTitleBar::layoutButtons() {
     int right(int(getFrame()->width()) - 2 * getFrame()->borderX() -
               (titleQ[pi] != null ? int(titleQ[pi]->width()) : 0));
 
-    if (titleButtonsLeft) {
-        for (const char *bc = titleButtonsLeft; *bc; bc++) {
+    const char* nearby = swapTitleButtons ? titleButtonsRight : titleButtonsLeft;
+    if (nearby) {
+        for (const char *bc = nearby; *bc; bc++) {
             if (*bc == ' ')
                 left++;
             else {
                 YFrameButton *b = getButton(*bc);
-                if (b && b->onRight() == false) {
+                if (b && isRight(b) == false) {
                     if (left + int(b->width()) >= right) {
                         b->hide();
                     } else {
@@ -230,13 +275,14 @@ void YFrameTitleBar::layoutButtons() {
         }
     }
 
-    if (titleButtonsRight) {
-        for (const char *bc = titleButtonsRight; *bc; bc++) {
+    const char* distant = swapTitleButtons ? titleButtonsLeft : titleButtonsRight;
+    if (distant) {
+        for (const char *bc = distant; *bc; bc++) {
             if (*bc == ' ')
                 right--;
             else {
                 YFrameButton *b = getButton(*bc);
-                if (b && b->onRight()) {
+                if (b && isRight(b)) {
                     if (right - int(b->width()) <= left) {
                         b->hide();
                     } else {
@@ -305,36 +351,42 @@ void YFrameTitleBar::paint(Graphics &g, const YRect &/*r*/) {
     if (titleQ[focused()] != null)
         onRight -= int(titleQ[focused()]->width());
 
-    if (titleButtonsLeft) {
-        for (const char *bc = titleButtonsLeft; *bc; bc++) {
+    const char* nearby = swapTitleButtons ? titleButtonsRight : titleButtonsLeft;
+    if (nearby) {
+        for (const char *bc = nearby; *bc; bc++) {
             if (*bc == ' ')
                 ++onLeft;
             else {
                 YFrameButton const *b(getButton(*bc));
-                if (b && b->visible() && b->onRight() == false)
+                if (b && b->visible() && isRight(b) == false)
                     onLeft = max(onLeft, (int)(b->x() + b->width()));
             }
         }
     }
 
-    if (titleButtonsRight) {
-        for (const char *bc = titleButtonsRight; *bc; bc++) {
+    const char* distant = swapTitleButtons ? titleButtonsLeft : titleButtonsRight;
+    if (distant) {
+        for (const char *bc = distant; *bc; bc++) {
             if (*bc == ' ')
                 --onRight;
             else {
                 YFrameButton const *b(getButton(*bc));
-                if (b && b->visible() && b->onRight())
+                if (b && b->visible() && isRight(b))
                     onRight = max(onRight - (int) b->width(), b->x());
             }
         }
     }
 
-    g.setFont(titleFont);
+    if (titleFont) {
+        g.setFont(titleFont);
+    }
 
     mstring title = getFrame()->getTitle();
-    int const yPos(int(height() - titleFont->height()) / 2 +
-                   titleFont->ascent() + titleBarVertOffset);
-    int tlen = title != null ? titleFont->textWidth(title) : 0;
+    int const fontHeight = titleFont ? titleFont->height() : 8;
+    int const fontAscent = titleFont ? titleFont->ascent() : 6;
+    int const yPos(int(height() - fontHeight) / 2 +
+                   fontAscent + titleBarVertOffset);
+    int tlen = title != null && titleFont ? titleFont->textWidth(title) : 0;
 
     int stringOffset(onLeft + (onRight - onLeft - tlen)
                      * (int) titleBarJustify / 100);
@@ -492,13 +544,14 @@ void YFrameTitleBar::renderShape(Graphics& g) {
         if (titleQ[focused()] != null)
             onRight -= int(titleQ[focused()]->width());
 
-        if (titleButtonsLeft)
-            for (const char *bc = titleButtonsLeft; *bc; bc++) {
+        const char* nearby = swapTitleButtons ? titleButtonsRight : titleButtonsLeft;
+        if (nearby)
+            for (const char *bc = nearby; *bc; bc++) {
                 if (*bc == ' ')
                     ++onLeft;
                 else {
                     YFrameButton const *b(getButton(*bc));
-                    if (b && b->visible() && b->onRight() == false) {
+                    if (b && b->visible() && isRight(b) == false) {
                         onLeft = max(onLeft, (int)(b->x() + b->width()));
 
                         ref<YPixmap> pixmap = b->getPixmap(0);
@@ -513,13 +566,14 @@ void YFrameTitleBar::renderShape(Graphics& g) {
                 }
             }
 
-        if (titleButtonsRight)
-            for (const char *bc = titleButtonsRight; *bc; bc++) {
+        const char* distant = swapTitleButtons ? titleButtonsLeft : titleButtonsRight;
+        if (distant)
+            for (const char *bc = distant; *bc; bc++) {
                 if (*bc == ' ')
                     --onRight;
                 else {
                     YFrameButton const *b(getButton(*bc));
-                    if (b && b->visible() && b->onRight()) {
+                    if (b && b->visible() && isRight(b)) {
                         onRight = max(onRight - (int) b->width(), b->x());
 
                         ref<YPixmap> pixmap = b->getPixmap(0);
@@ -538,7 +592,7 @@ void YFrameTitleBar::renderShape(Graphics& g) {
         onRight += x();
 
         mstring title = getFrame()->getTitle();
-        int tlen = title != null ? titleFont->textWidth(title) : 0;
+        int tlen = title != null && titleFont ? titleFont->textWidth(title) : 0;
         bool const pi(focused());
 
         if (titleL[pi] != null) {
