@@ -13,6 +13,7 @@
 #include "wmmgr.h"
 #include "wmapp.h"
 #include "workspaces.h"
+#include "wpixmaps.h"
 #include <assert.h>
 #include "intl.h"
 
@@ -70,65 +71,66 @@ void WindowListBox::activateItem(YListItem *item) {
     ClientData *f = i->getFrame();
     if (f) {
         f->activateWindow(true, false);
-        windowList->getFrame()->wmHide();
+        windowList->handleClose();
     } else {
         int w = i->getWorkspace();
         if (w >= 0) {
             manager->activateWorkspace(w);
-            windowList->getFrame()->wmHide();
+            windowList->handleClose();
         }
     }
 }
 
-void WindowListBox::getSelectedWindows(YArray<YFrameWindow *> &frames) {
+YArrange WindowListBox::getSelectedWindows() {
+    YFrameWindow** frames = nullptr;
+    int count = 0;
     if (hasSelection()) {
+        for (IterType iter(getIterator()); ++iter; ) {
+            count += isSelected(iter.where());
+        }
+    }
+    if (count) {
+        frames = new YFrameWindow*[count];
+        int k = 0;
         for (IterType iter(getIterator()); ++iter; ) {
             if (isSelected(iter.where())) {
                 WindowListItem *item = static_cast<WindowListItem *>(*iter);
                 ClientData *frame = item->getFrame();
-                if (frame)
-                    frames.append(static_cast<YFrameWindow *>(frame));
+                if (frame && k < count) {
+                    frames[k++] = static_cast<YFrameWindow *>(frame);
+                }
             }
         }
     }
+    return YArrange(frames, count);
 }
 
 void WindowListBox::actionPerformed(YAction action, unsigned int modifiers) {
     bool save = focusCurrentWorkspace;
     if (save) focusCurrentWorkspace = false;
 
-    YArray<YFrameWindow *> frameList;
-    getSelectedWindows(frameList);
-
-    if (action == actionTileVertical ||
-        action == actionTileHorizontal)
-    {
-        if (frameList.nonempty())
-            manager->tileWindows(frameList.getItemPtr(0),
-                                 frameList.getCount(),
-                                 (action == actionTileVertical));
-    } else if (action == actionCascade ||
-               action == actionArrange)
-    {
-        if (frameList.nonempty()) {
-            if (action == actionCascade) {
-                manager->cascadePlace(frameList.getItemPtr(0),
-                                      frameList.getCount());
-            } else if (action == actionArrange) {
-                manager->smartPlace(frameList.getItemPtr(0),
-                                    frameList.getCount());
+    YArrange arrange = getSelectedWindows();
+    if (arrange == false) {
+    }
+    else if (action == actionTileVertical) {
+        manager->tileWindows(arrange, true);
+    }
+    else if (action == actionTileHorizontal) {
+        manager->tileWindows(arrange, false);
+    }
+    else if (action == actionCascade) {
+        manager->cascadePlace(arrange);
+    }
+    else if (action == actionArrange) {
+        manager->smartPlace(arrange);
+    }
+    else {
+        for (YFrameWindow* frame : arrange) {
+            if ((action != actionHide || !frame->isHidden()) &&
+                (action != actionMinimize || !frame->isMinimized())) {
+                frame->actionPerformed(action, modifiers);
             }
         }
-    } else {
-        for (int i = 0; i < frameList.getCount(); i++) {
-            if (action == actionHide)
-                if (frameList[i]->isHidden())
-                    continue;
-            if (action == actionMinimize)
-                if (frameList[i]->isMinimized())
-                    continue;
-            frameList[i]->actionPerformed(action, modifiers);
-        }
     }
 
     if (save) focusCurrentWorkspace = save;
@@ -141,7 +143,7 @@ bool WindowListBox::handleKey(const XKeyEvent &key) {
 
         switch (k) {
         case XK_Escape:
-            windowList->getFrame()->wmHide();
+            windowList->handleClose();
             return true;
         case XK_F10:
         case XK_Menu:
@@ -156,7 +158,8 @@ bool WindowListBox::handleKey(const XKeyEvent &key) {
                                            YPopupWindow::pfPopupMenu);
                 } else {
                     YMenu* windowListAllPopup = windowList->getWindowListAllPopup();
-                    windowListAllPopup->popup(nullptr, nullptr, nullptr, key.x_root, key.y_root,
+                    windowListAllPopup->popup(nullptr, nullptr, nullptr,
+                                              key.x_root, key.y_root,
                                               YPopupWindow::pfCanFlipVertical |
                                               YPopupWindow::pfCanFlipHorizontal |
                                               YPopupWindow::pfPopupMenu);
@@ -177,7 +180,7 @@ bool WindowListBox::handleKey(const XKeyEvent &key) {
 }
 
 void WindowListBox::handleClick(const XButtonEvent &up, int count) {
-    if (up.button == 3 && count == 1 && IS_BUTTON(up.state, Button3Mask)) {
+    if (up.button == 3 && count == 1 && xapp->isButton(up.state, Button3Mask)) {
         int no = findItemByPoint(up.x, up.y);
 
         if (no != -1) {
@@ -196,7 +199,8 @@ void WindowListBox::handleClick(const XButtonEvent &up, int count) {
                                    YPopupWindow::pfPopupMenu);
         } else {
             YMenu* windowListAllPopup = windowList->getWindowListAllPopup();
-            windowListAllPopup->popup(nullptr, nullptr, nullptr, up.x_root, up.y_root,
+            windowListAllPopup->popup(nullptr, nullptr, nullptr,
+                                      up.x_root, up.y_root,
                                       YPopupWindow::pfCanFlipVertical |
                                       YPopupWindow::pfCanFlipHorizontal |
                                       YPopupWindow::pfPopupMenu);
@@ -320,21 +324,30 @@ void WindowListBox::enableCommands(YMenu *popup) {
         }
     }
     if (selected == false) {
-        moveMenu->disableCommand(actionNull);
         popup->disableCommand(actionNull);
     }
     else {
         moveMenu->setActionListener(this);
+        tileMenu->setActionListener(this);
         layerMenu->setActionListener(this);
     }
 }
 
 WindowList::WindowList(YWindow *aParent):
     YFrameClient(aParent, nullptr),
+    allWorkspacesItem(nullptr),
     scroll(new YScrollView(this)),
     list(new WindowListBox(scroll, scroll))
 {
     addStyle(wsNoExpose);
+
+    if (listbackPixmap != null) {
+        scroll->setBackgroundPixmap(listbackPixmap);
+    }
+    else if (nonempty(clrListBox)) {
+        scroll->setBackground(YColorName(clrListBox));
+    }
+
     scroll->setView(list);
 
     updateWorkspaces();
@@ -371,6 +384,7 @@ WindowListPopup::WindowListPopup() {
     addItem(_("_Raise"), -2, KEY_NAME(gKeyWinRaise), actionRaise);
     addItem(_("_Lower"), -2, KEY_NAME(gKeyWinLower), actionLower);
     addSubmenu(_("La_yer"), -2, layerMenu);
+    addSubmenu(_("Tile"), -2, tileMenu);
     addSeparator();
     addSubmenu(_("Move _To"), -2, moveMenu);
     addItem(_("Occupy _All"), -2, KEY_NAME(gKeyWinOccupyAll), actionOccupyAllOrCurrent);
@@ -393,8 +407,8 @@ WindowListPopup::WindowListPopup() {
     closeSubmenu->addSeparator();
     closeSubmenu->addItem(_("_Kill Client"), -2, null, actionKill);
 #if 0
-    closeSubmenu->addItem(_("_Terminate Process"), -2, 0, actionTermProcess)->setEnabled(false);
-    closeSubmenu->addItem(_("Kill _Process"), -2, 0, actionKillProcess)->setEnabled(false);
+    closeSubmenu->addItem(_("_Terminate Process"), -2, 0, actionTermProcess);
+    closeSubmenu->addItem(_("Kill _Process"), -2, 0, actionKillProcess);
 #endif
 
     addItem(_("_Close"), -2, actionClose, closeSubmenu);
@@ -445,13 +459,13 @@ WindowList::~WindowList() {
 
 void WindowList::updateWorkspaces() {
     if (workspaceItem.isEmpty()) {
-        workspaceItem.append(new WindowListItem(nullptr, AllWorkspaces));
-        list->addItem(workspaceItem[0]);
+        allWorkspacesItem = new WindowListItem(nullptr, AllWorkspaces);
+        workspaceItem.append(allWorkspacesItem);
+        list->addItem(allWorkspacesItem);
     }
-    WindowListItem *allWS = workspaceItem[workspaceItem.getCount() - 1];
     for (int ws = workspaceItem.getCount() - 1; ws < workspaceCount; ++ws) {
         workspaceItem.insert(ws, new WindowListItem(nullptr, ws));
-        list->addBefore(allWS, workspaceItem[ws]);
+        list->addBefore(allWorkspacesItem, workspaceItem[ws]);
     }
     for (int ws = workspaceItem.getCount() - 1; ws > workspaceCount; --ws) {
         list->removeItem(workspaceItem[ws - 1]);
@@ -486,12 +500,63 @@ void WindowList::insertApp(WindowListItem *item) {
         frame->owner()->winListItem())
     {
         list->addAfter(frame->owner()->winListItem(), item);
-    } else {
+    }
+    else {
         int ws = frame->getWorkspace();
-        if (0 <= ws && ws < workspaceItem.getCount())
-            list->addAfter(workspaceItem[ws], item);
-        else
+        Window lead = frame->clientLeader();
+        ClassHint* hint = frame->classHint();
+        int start = (0 <= ws && ws + 1 < workspaceItem.getCount())
+                    ? list->findItem(workspaceItem[ws])
+                    : list->findItem(allWorkspacesItem);
+        int stop = (0 <= ws && ws + 1 < workspaceItem.getCount())
+                    ? list->findItem(workspaceItem[ws + 1])
+                    : list->getItemCount();
+        if (0 <= start && start < stop) {
+            int best = 1 + start;
+            if (lead) {
+                int have = 0;
+                for (int i = best; i < stop; ++i) {
+                    WindowListItem* test =
+                        dynamic_cast<WindowListItem*>(list->getItem(i));
+                    if (test && test->getFrame() &&
+                        lead == test->getFrame()->clientLeader()) {
+                        have = i + 1;
+                    }
+                }
+                if (have) {
+                    list->insertAt(have, item);
+                    return;
+                }
+            }
+            int have = stop;
+            for (int i = best; i < stop; ++i) {
+                WindowListItem* test =
+                    dynamic_cast<WindowListItem*>(list->getItem(i));
+                if (test && test->getFrame()) {
+                    ClassHint* klas = test->getFrame()->classHint();
+                    int cmp =
+                        hint == nullptr ? +1 :
+                        klas == nullptr ? -1 :
+                        nonempty(hint->res_class) ?
+                        isEmpty(klas->res_class) ? -1 :
+                        strcmp(hint->res_class, klas->res_class) :
+                        nonempty(hint->res_name) ?
+                        isEmpty(klas->res_name) ? -1 :
+                        strcmp(hint->res_name, klas->res_name) : +1;
+                    if (cmp < 0) {
+                        list->insertAt(i, item);
+                        return;
+                    }
+                    if (cmp == 0) {
+                        have = i + 1;
+                    }
+                }
+            }
+            list->insertAt(have, item);
+        }
+        else {
             list->addItem(item);
+        }
     }
 }
 
@@ -543,7 +608,7 @@ void WindowList::showFocused(int x, int y) {
         int x = int((dw - w) / 2);
         int y = int((dh - h) / 2);
         setGeometry(YRect(x, y, w, h));
-        manager->manageClient(handle(), false);
+        manager->manageClient(this);
     }
     if (getFrame()) {
         if (x == -1 && y == -1) {
@@ -579,7 +644,21 @@ void WindowList::showFocused(int x, int y) {
         }
         getFrame()->setRequestedLayer(WinLayerAboveDock);
         getFrame()->setAllWorkspaces();
-        getFrame()->activateWindow(true);
+        getFrame()->wmRaise();
+        getFrame()->wmShow();
+        xapp->sync();
+        focusTimer.setTimer(None, this, true);
+    }
+}
+
+bool WindowList::handleTimer(YTimer* timer) {
+    if (timer == &focusTimer) {
+        if (visible()) {
+            manager->setFocus(getFrame());
+        }
+        return false;
+    } else {
+        return YFrameClient::handleTimer(timer);
     }
 }
 
