@@ -13,8 +13,8 @@
  *  - introduced YStringArray
  */
 
-#ifndef __YARRAY_H
-#define __YARRAY_H
+#ifndef YARRAY_H
+#define YARRAY_H
 
 #include <string.h>
 #include "base.h"
@@ -34,8 +34,8 @@ public:
     typedef int SizeType;
     typedef unsigned char StorageType;
 
-    explicit YBaseArray(SizeType elementSize):
-        fElementSize(elementSize), fCapacity(0), fCount(0), fElements(0) {}
+    explicit YBaseArray(unsigned elementSize):
+        fElementSize(elementSize), fCapacity(0), fCount(0), fElements(nullptr) {}
     YBaseArray(YBaseArray &other);
     YBaseArray(const YBaseArray& other);
     virtual ~YBaseArray() { clear(); }
@@ -58,33 +58,26 @@ public:
 
 protected:
     const StorageType *getElement(const SizeType index) const {
-        return fElements + (index * fElementSize);
+        return fElements + unsigned(index) * fElementSize;
     }
     StorageType *getElement(const SizeType index) {
-        return fElements + (index * fElementSize);
+        return fElements + unsigned(index) * fElementSize;
     }
 
-    const void *getBegin() const { return getElement(0); }
-    const void *getEnd() const { return getElement(getCount()); }
+    const void* begin() const { return fElements; }
+    const void* end() const { return getElement(getCount()); }
 
     void release();
     void swap(YBaseArray& other);
 
 public:
-    SizeType getIndex(void const * ptr) const {
-        PRECONDITION(ptr >= getBegin() && ptr < getEnd());
-        return (ptr >= getBegin() && ptr < getEnd()
-             ? SizeType(static_cast<const StorageType *>(ptr) - fElements)
-                 / fElementSize
-             : npos);
-    }
     const void *getItem(const SizeType index) const {
         PRECONDITION(index < getCount());
-        return (index < getCount() ? getElement(index) : 0);
+        return (index < getCount() ? getElement(index) : nullptr);
     }
     void *getItem(const SizeType index) {
         PRECONDITION(index < getCount());
-        return (index < getCount() ? getElement(index) : 0);
+        return (index < getCount() ? getElement(index) : nullptr);
     }
 
     const void *operator[](const SizeType index) const {
@@ -98,7 +91,7 @@ public:
 private:
     void operator=(const YBaseArray&); // not implemented
 
-    const SizeType fElementSize;
+    const unsigned fElementSize;
     SizeType fCapacity, fCount;
     StorageType *fElements;
 };
@@ -127,6 +120,9 @@ public:
     void insert(const SizeType index, const DataType &item) {
         YBaseArray::insert(index, &item);
     }
+    void pop() {
+        remove(getCount() - 1);
+    }
 
     const DataType *getItemPtr(const SizeType index) const {
         return static_cast<const DataType *>(YBaseArray::getItem(index));
@@ -135,10 +131,10 @@ public:
         return *getItemPtr(index);
     }
     const DataType &operator[](const SizeType index) const {
-        return getItem(index);
+        return *getItemPtr(index);
     }
-    const DataType &operator*() const {
-        return getItem(0);
+    const DataType &last() const {
+        return *getItemPtr(getCount() - 1);
     }
 
     DataType *getItemPtr(const SizeType index) {
@@ -148,11 +144,15 @@ public:
         return *getItemPtr(index);
     }
     DataType &operator[](const SizeType index) {
-        return getItem(index);
+        return *getItemPtr(index);
     }
     DataType &operator*() {
-        return getItem(0);
+        return *getItemPtr(0);
     }
+    DataType &last() {
+        return *getItemPtr(getCount() - 1);
+    }
+
     YArray<DataType>& operator+=(const DataType& item) {
         append(item); return *this;
     }
@@ -162,6 +162,12 @@ public:
     void swap(YArray<DataType>& other) {
         YBaseArray::swap(other);
     }
+    const DataType* begin() const {
+        return (const DataType*) YBaseArray::begin();
+    }
+    const DataType* end() const {
+        return (const DataType*) YBaseArray::end();
+    }
     IterType iterator();
     IterType reverseIterator();
 };
@@ -179,6 +185,7 @@ public:
     using BaseType::getItem;
 
     YObjectArray() {}
+    YObjectArray(SizeType capacity) : BaseType(capacity) {}
     virtual ~YObjectArray() {
         clear();
     }
@@ -206,6 +213,13 @@ public:
         BaseType::append(item); return *this;
     }
 
+    DataType** begin() const {
+        return (DataType**) YBaseArray::begin();
+    }
+    DataType** end() const {
+        return (DataType**) YBaseArray::end();
+    }
+
 private:
     YObjectArray(const YObjectArray&);
     YObjectArray& operator=(const YObjectArray&);
@@ -235,7 +249,7 @@ public:
         return *getItemPtr(index);
     }
     ref<DataType> operator[](const SizeType index) const {
-        return getItem(index);
+        return *getItemPtr(index);
     }
 
     virtual void remove(const SizeType index) {
@@ -257,6 +271,22 @@ public:
         YBaseArray::shrink(reducedCount);
     }
 
+    const ref<DataType>* begin() const {
+        return (const ref<DataType>*) YBaseArray::begin();
+    }
+    const ref<DataType>* end() const {
+        return (const ref<DataType>*) YBaseArray::end();
+    }
+
+    int find(DataType* data) {
+        for (SizeType i = 0; i < getCount(); ++i) {
+            if (data == getItemPtr(i)->_ptr()) {
+                return i;
+            }
+        }
+        return npos;
+    }
+
 private:
     ref<DataType>* getItemPtr(const SizeType index) const {
         return static_cast<ref<DataType> *>(
@@ -317,47 +347,11 @@ public:
     char **release();
 };
 
-/*******************************************************************************
- * A stack emulated by a dynamic array
- ******************************************************************************/
-
-template <class DataType>
-class YStack: public YArray<DataType> {
-public:
-    using YArray<DataType>::getCount;
-    using YArray<DataType>::nonempty;
-
-    const DataType &getTop() const {
-        PRECONDITION(nonempty());
-        return getItem(getCount() - 1);
-    }
-    const DataType &operator*() const { return getTop(); }
-
-    virtual void push(const DataType &item) { append(item); }
-    void pop() {
-        PRECONDITION(nonempty());
-        remove(getCount() - 1);
-    }
-};
-
-/*******************************************************************************
- * A set emulated by a stack
- ******************************************************************************/
-
-template <class DataType>
-class YStackSet: public YStack<DataType> {
-public:
-    virtual void push(const DataType &item) {
-        findRemove(*this, item);
-        YStack<DataType>::push(item);
-    }
-};
-
 /*******************************************************************************
  * An array of mstrings
  ******************************************************************************/
 
-#ifdef __MSTRING_H
+#ifdef MSTRING_H
 class MStringArray: public YArray<mstring> {
 public:
     typedef YArray<mstring> BaseType;
@@ -400,9 +394,9 @@ public:
         return *getItemPtr(index);
     }
     mstring& operator[](const SizeType index) const {
-        return getItem(index);
+        return *getItemPtr(index);
     }
-    MStringArray& operator+=(mstring& item) {
+    MStringArray& operator+=(const mstring& item) {
         append(item); return *this;
     }
 
@@ -427,13 +421,20 @@ public:
 
     void sort();
 
+    mstring* begin() const {
+        return const_cast<mstring*>(BaseType::begin());
+    }
+    mstring* end() const {
+        return const_cast<mstring*>(BaseType::end());
+    }
+
 private:
     mstring* getItemPtr(const SizeType index) const {
         return static_cast<mstring *>(
                 const_cast<void *>(YBaseArray::getItem(index)));
     }
 };
-#endif  /*__MSTRING_H*/
+#endif /* MSTRING_H */
 
 /*******************************************************************************
  * An associative array
@@ -458,8 +459,8 @@ public:
         return hash < hash2 ? -1 :
                hash > hash2 ? +1 :
                key == key2 ? 0 :
-               key == 0 && key2 != 0 ? -1 :
-               key != 0 && key2 == 0 ? +1 :
+               key == nullptr && key2 != nullptr ? -1 :
+               key != nullptr && key2 == nullptr ? +1 :
                strcmp(key, key2);
     }
 
@@ -710,26 +711,24 @@ YArrayIterator<DataType> YArray<DataType>::reverseIterator() {
 
 template<class DataType>
 int find(YArray<DataType>& array, DataType& data) {
-    YArrayIterator<DataType> iter = array.iterator();
-    while (++iter)
-        if (*iter == data) return iter.where();
-    return -1;
-}
-
-template<class DataType>
-int find(const YArray<DataType>& array, const DataType& data) {
-    for (YBaseArray::SizeType i = 0; i < array.getCount(); ++i)
+    for (YBaseArray::SizeType i = 0, n = array.getCount(); i < n; ++i)
         if (array[i] == data) return i;
     return -1;
 }
 
 template<class DataType>
-bool findRemove(YArray<DataType>& array, DataType& data) {
-    int k = find(array, data);
-    if (k >= 0)
-        array.remove(k);
-    return k >= 0;
+int find(const YArray<DataType>& array, const DataType& data) {
+    for (YBaseArray::SizeType i = 0, n = array.getCount(); i < n; ++i)
+        if (array[i] == data) return i;
+    return -1;
 }
+
+template<class DataType>
+bool findRemove(YArray<DataType>& array, DataType data) {
+    int k = find(array, data);
+    return 0 <= k ? array.remove(k), true : false;
+}
+
 #endif
 
 // vim: set sw=4 ts=4 et:
