@@ -50,11 +50,11 @@ public:
         if (sizeof(unsigned long) < 8) {
             return (r & 0xFF00) << 16 | (g & 0xFF00) << 8 | ((b >> 8) & 0xFF)
                  | ((a & 0xFF) << 24);
-	}
+        }
         else {
             return (uint64_t) r << 32 | (uint64_t) g << 16 | b
                  | (uint64_t) (a & 0xFF) << 48;
-	}
+        }
     }
     unsigned short red() const {
         if (sizeof(unsigned long) < 8)
@@ -258,6 +258,16 @@ void YColor::alloc(const char* name) {
     fPixel = cache.black();
 }
 
+YColor& YColor::reverse() {
+    if (fPixel) {
+        fPixel = cache.get(0xFFFF - fPixel->red(),
+                           0xFFFF - fPixel->green(),
+                           0xFFFF - fPixel->blue(),
+                           fPixel->alpha());
+    }
+    return *this;
+}
+
 #ifdef CONFIG_XFREETYPE
 XftColor* YPixel::allocXft() {
     fXftColor = new XftColor;
@@ -382,15 +392,15 @@ YColor YPixel::brighter() {
 }
 
 unsigned long YColor::pixel() {
-    return fPixel->pixel();
+    return fPixel ? fPixel->pixel() : xapp->black();
 }
 
 YColor YColor::darker() {
-    return fPixel->darker();
+    return fPixel ? fPixel->darker() : *this;
 }
 
 YColor YColor::brighter() {
-    return fPixel->brighter();
+    return fPixel ? fPixel->brighter() : *this;
 }
 
 bool YColor::operator==(YColor& c) {
