@@ -11,16 +11,16 @@
 
 #ifdef CFGDEF
 #define XFV(t,a,b,c) \
-    t a(b); \
-    t a##Xft = c;
+    t a##Core = b; \
+    t a##Xft = c; \
+    YFontName a(&a##Core, &a##Xft);
 #else
 #define XFV(t,a,b,c) \
-    extern t a; \
-    extern t a##Xft;
+    extern t a##Core; \
+    extern t a##Xft; \
+    extern YFontName a;
 #endif
 
-#define XFA(a) a, a##Xft
-
 #ifdef CFGDEF
 #define XIV(t,a,b) t a(b);
 #else
@@ -31,25 +31,7 @@
 #define YCONFIG_H
 
 #include <X11/X.h>
-
-#ifdef CONFIG_XFREETYPE
-#define FONT(pt) "-*-sans-medium-r-*-*-*-" #pt "-*-*-*-*-*-*"
-#define BOLDFONT(pt) "-*-sans-bold-r-*-*-*-" #pt "-*-*-*-*-*-*"
-#define TTFONT(pt) "-*-monospace-medium-r-*-*-*-" #pt "-*-*-*-*-*-*"
-#define BOLDTTFONT(pt) "-*-monospace-bold-r-*-*-*-" #pt "-*-*-*-*-*-*"
-#else
-#ifdef FONTS_ADOBE
-#define FONT(pt) "-b&h-lucida-medium-r-*-*-*-" #pt "-*-*-*-*-*-*"
-#define BOLDFONT(pt) "-b&h-lucida-bold-r-*-*-*-" #pt "-*-*-*-*-*-*"
-#define TTFONT(pt) "-b&h-lucidatypewriter-medium-r-*-*-*-" #pt "-*-*-*-*-*-*"
-#define BOLDTTFONT(pt) "-b&h-lucidatypewriter-bold-r-*-*-*-" #pt "-*-*-*-*-*-*"
-#else
-#define FONT(pt) "-adobe-helvetica-medium-r-*-*-*-" #pt "-*-*-*-*-*-*"
-#define BOLDFONT(pt) "-adobe-helvetica-bold-r-*-*-*-" #pt "-*-*-*-*-*-*"
-#define TTFONT(pt) "-adobe-courier-medium-r-*-*-*-" #pt "-*-*-*-*-*-*"
-#define BOLDTTFONT(pt) "-adobe-courier-bold-r-*-*-*-" #pt "-*-*-*-*-*-*"
-#endif
-#endif
+#include "yfontname.h"
 
 #define kfShift  1
 #define kfCtrl   2
@@ -73,14 +55,14 @@ struct WMKey {
 #ifdef CFGDESC
 #define DESC(d) d
 #else
-#define DESC(d) ((const char *) 0)
+#define DESC(d) nullptr
 #endif
 
 #define OBV(n,v,d)     cfoption(n, sizeof(n), v, DESC(d))
 #define OIV(n,v,m,M,d) cfoption(n, sizeof(n), v, m, M, DESC(d))
 #define OUV(n,v,m,M,d) cfoption(n, sizeof(n), v, m, M, DESC(d))
 #define OSV(n,v,d)     cfoption(n, sizeof(n), v, DESC(d))
-#define OFV(n,v,d)     cfoption(n, sizeof(n), v, DESC(d)), \
+#define OFV(n,v,d)     cfoption(n, sizeof(n), v##Core, DESC(d)), \
                        cfoption(n "Xft", sizeof(n "Xft"), v##Xft, DESC(d))
 #define OKV(n,v,d)     cfoption(n, sizeof(n), &v, DESC(d))
 #define OKF(n,f,d)     cfoption(n, sizeof(n), f, DESC(d))
@@ -133,12 +115,19 @@ struct cfoption {
         v.f.notify = f;
     }
     cfoption() : type(CF_NONE), size(0), name(nullptr), description(nullptr) { }
-    bool boolval() { return *v.b.bool_value; }
-    int intval() { return *v.i.int_value; }
-    unsigned uintval() { return *v.u.uint_value; }
-    const char* str() { return *v.s.string_value; }
-    WMKey* key() { return v.k.key_value; }
-    notifyfun fun() { return v.f.notify; }
+    bool boolval() const { return *v.b.bool_value; }
+    int intval() const { return *v.i.int_value; }
+    int intmax() const { return v.i.max; }
+    int intmin() const { return v.i.min; }
+    unsigned uintval() const { return *v.u.uint_value; }
+    unsigned uintmax() const { return v.u.max; }
+    unsigned uintmin() const { return v.u.min; }
+    const char* str() const { return *v.s.string_value; }
+    WMKey* key() const { return v.k.key_value; }
+    notifyfun fun() const { return v.f.notify; }
+
+    bool operator==(const cfoption& r) const;
+    bool operator!=(const cfoption& r) const { return !operator==(r); }
 };
 
 class Argument;
@@ -157,7 +146,7 @@ public:
     static void freeConfig(cfoption* options);
     static char* getArgument(Argument* dest, char* p, bool comma = false);
     static bool loadConfigFile(cfoption* options, upath fileName,
-                               cfoption* more = nullptr);
+                               cfoption* more = nullptr, cfoption* xtra = nullptr);
     static bool findLoadConfigFile(cfoption* options, const char* name);
     static bool findLoadThemeFile(cfoption* options);
     static bool parseKey(const char* arg, KeySym* key, unsigned int* mod);
