@@ -31,7 +31,6 @@ public:
 
 YInputLine::YInputLine(YWindow *parent, YInputListener *listener):
     YWindow(parent),
-    fText(null),
     markPos(0),
     curPos(0),
     leftOfs(0),
@@ -41,7 +40,7 @@ YInputLine::YInputLine(YWindow *parent, YInputListener *listener):
     fSelecting(false),
     fBlinkTime(333),
     fListener(listener),
-    inputFont(YFont::getFont(XFA(inputFontName))),
+    inputFont(inputFontName),
     inputBg(&clrInput),
     inputFg(&clrInputText),
     inputSelectionBg(&clrInputSelection),
@@ -55,7 +54,7 @@ YInputLine::YInputLine(YWindow *parent, YInputListener *listener):
 YInputLine::~YInputLine() {
 }
 
-void YInputLine::setText(const mstring &text, bool asMarked) {
+void YInputLine::setText(mstring text, bool asMarked) {
     fText = text;
     leftOfs = 0;
     curPos = fText.length();
@@ -81,7 +80,7 @@ void YInputLine::configure(const YRect2& r) {
 }
 
 void YInputLine::paint(Graphics &g, const YRect &/*r*/) {
-    ref<YFont> font = inputFont;
+    YFont font = inputFont;
     int min, max, minOfs = 0, maxOfs = 0;
     int textLen = fText.length();
 
@@ -97,8 +96,8 @@ void YInputLine::paint(Graphics &g, const YRect &/*r*/) {
         g.setColor(inputBg);
         g.fillRect(0, 0, width(), height());
     } else {
-        minOfs = font->textWidth(fText.substring(0, min)) - leftOfs;
-        maxOfs = font->textWidth(fText.substring(0, max)) - leftOfs;
+        minOfs = font->textWidth(fText.data(), min) - leftOfs;
+        maxOfs = font->textWidth(fText.data(), max) - leftOfs;
 
         if (minOfs > 0) {
             g.setColor(inputBg);
@@ -116,31 +115,32 @@ void YInputLine::paint(Graphics &g, const YRect &/*r*/) {
     }
 
     if (font != null) {
-        int yp = 1 + font->ascent();
-        int curOfs = font->textWidth(fText.substring(0, curPos));
-        int cx = curOfs - leftOfs;
+        int yo = ::max(0, (int(height()) - int(font->height())) / 2);
+        int yp = font->ascent() + yo;
+        int curOfs = font->textWidth(fText.data(), curPos);
+        int cx = ::max(1, curOfs - leftOfs);
 
         g.setFont(font);
 
         if (curPos == markPos || !fHasFocus || fText == null) {
             g.setColor(inputFg);
             if (fText != null)
-                g.drawChars(fText.substring(0, textLen), -leftOfs, yp);
+                g.drawChars(fText.data(), 0, textLen, -leftOfs, yp);
             if (fHasFocus && fCursorVisible)
-                g.drawLine(cx, 0, cx, font->height() + 2);
+                g.drawLine(cx, yo, cx, font->height() + 2);
         } else {
             if (min > 0) {
                 g.setColor(inputFg);
-                g.drawChars(fText.substring(0, min), -leftOfs, yp);
+                g.drawChars(fText.data(), 0, min, -leftOfs, yp);
             }
             /// !!! same here
             if (min < max) {
                 g.setColor(inputSelectionFg);
-                g.drawChars(fText.substring(min, max - min), minOfs, yp);
+                g.drawChars(fText.data(), min, max - min, minOfs, yp);
             }
             if (max < textLen) {
                 g.setColor(inputFg);
-                g.drawChars(fText.substring(max, textLen - max), maxOfs, yp);
+                g.drawChars(fText.data(), max, textLen - max, maxOfs, yp);
             }
         }
     }
@@ -327,7 +327,7 @@ bool YInputLine::handleKey(const XKeyEvent &key) {
                 char s[16];
 
                 if (getCharFromEvent(key, s, sizeof(s))) {
-                    replaceSelection(mstring(s, strlen(s)));
+                    replaceSelection(s, strlen(s));
                     return true;
                 }
             }
@@ -429,19 +429,19 @@ void YInputLine::handleSelection(const XSelectionEvent &selection) {
         YProperty prop(selection.requestor, selection.property,
                        F8, 32 * 1024, selection.target, True);
         if (prop) {
-            replaceSelection(mstring(prop.data<char>(), prop.size()));
+            replaceSelection(prop.data<char>(), prop.size());
         }
     }
 }
 
 unsigned YInputLine::offsetToPos(int offset) {
-    ref<YFont> font = inputFont;
+    YFont font = inputFont;
     int ofs = 0, pos = 0;
     int textLen = fText.length();
 
     if (font != null) {
         while (pos < textLen) {
-            ofs += font->textWidth(fText.substring(pos, 1));
+            ofs += font->textWidth(fText.data() + pos, 1);
             if (ofs < offset)
                 pos++;
             else
@@ -526,10 +526,10 @@ void YInputLine::limit() {
     if (markPos > textLen)
         markPos = textLen;
 
-    ref<YFont> font = inputFont;
+    YFont font = inputFont;
     if (font != null) {
-        int curOfs = font->textWidth(fText.substring(0, curPos));
-        int curLen = font->textWidth(fText.substring(0, textLen));
+        int curOfs = font->textWidth(fText.data(), curPos);
+        int curLen = font->textWidth(fText.data(), textLen);
 
         if (curOfs >= leftOfs + int(width()) + 1)
             leftOfs = curOfs - width() + 2;
@@ -542,18 +542,19 @@ void YInputLine::limit() {
     }
 }
 
-void YInputLine::replaceSelection(const mstring &str) {
+void YInputLine::replaceSelection(const char* insert, int amount) {
     unsigned from = min(curPos, markPos);
     unsigned to = max(curPos, markPos);
-    fText = fText.replace(from, to - from, str);
-    curPos = markPos = from + str.length();
+    YWideString wide(insert, amount);
+    fText.replace(from, to - from, wide);
+    curPos = markPos = from + wide.length();
     limit();
     repaint();
 }
 
 bool YInputLine::deleteSelection() {
     if (hasSelection()) {
-        replaceSelection(null);
+        replaceSelection("", 0);
         return true;
     }
     return false;
@@ -579,12 +580,6 @@ bool YInputLine::deletePreviousChar() {
     return false;
 }
 
-bool YInputLine::insertChar(char ch) {
-    char s[2] = { ch, 0 };
-    replaceSelection(s);
-    return true;
-}
-
 #define CHCLASS(c) ((c) == ' ')
 
 unsigned YInputLine::nextWord(unsigned p, bool sep) {
@@ -670,9 +665,14 @@ bool YInputLine::cutSelection() {
 }
 
 bool YInputLine::copySelection() {
-    int min = ::min(curPos, markPos), max = ::max(curPos, markPos);
-    return min < max && fText.nonempty()
-        && (xapp->setClipboardText(fText.substring(min, max - min)), true);
+    unsigned min = ::min(curPos, markPos), max = ::max(curPos, markPos);
+    if (min < max && fText.length() <= max) {
+        YWideString copy(fText.copy(min, max - min));
+        xapp->setClipboardText(copy);
+        return true;
+    } else {
+        return false;
+    }
 }
 
 void YInputLine::actionPerformed(YAction action, unsigned int /*modifiers*/) {
@@ -695,7 +695,8 @@ void YInputLine::autoScroll(int delta, const XMotionEvent *motion) {
 
 void YInputLine::complete() {
     char* res = nullptr;
-    int res_count = globit_best(fText, &res, nullptr, nullptr);
+    mstring mstr(fText);
+    int res_count = globit_best(mstr, &res, nullptr, nullptr);
     // directory is not a final match
     if (res_count == 1 && upath(res).dirExists())
         res_count++;
