@@ -2,6 +2,7 @@
 #define YPREFS_H
 
 #include "yconfig.h"
+#include "fontmacro.h"
 
 XIV(bool, dontRotateMenuPointer,                true)
 XIV(bool, fontPreferFreetype,                   true)
@@ -9,16 +10,18 @@ XIV(bool, menuMouseTracking,                    false)
 XIV(bool, replayMenuCancelClick,                false)
 XIV(bool, showPopupsAbovePointer,               false)
 XIV(bool, showEllipsis,                         true)
+XIV(bool, rightToLeft,                          false)
+XIV(bool, leftToRight,                          true)
 #ifdef CONFIG_I18N
 XIV(bool, multiByte,                            true)
 #endif
-XIV(bool, modSuperIsCtrlAlt,                    true)
+XIV(bool, modSuperIsCtrlAlt,                    false)
 XIV(bool, doubleBuffer,                         true)
 XIV(bool, xrrDisable,                           false)
 XIV(int, xineramaPrimaryScreen,                 0)
 XIV(int, MenuActivateDelay,                     40)
 XIV(int, SubmenuActivateDelay,                  300)
-XIV(int, DelayFuzziness,                        10)
+extern int DelayFuzziness;
 XIV(int, ClickMotionDistance,                   4)
 XIV(int, ClickMotionDelay,                      200)
 XIV(int, MultiClickTime,                        400)
@@ -57,7 +60,7 @@ XIV(WMLook, wmLook,                             CONFIG_DEFAULT_LOOK)
 
 XSV(const char *, clrToolTip,                   "rgb:E0/E0/00")
 XSV(const char *, clrToolTipText,               "rgb:00/00/00")
-XFV(const char *, toolTipFontName,              FONT(120), "sans-serif:size=12")
+XFV(const char *, toolTipFontName,              FONT(12,120), FSANS)
 
 #endif
 
