@@ -1,15 +1,16 @@
 #include "config.h"
 
-// this has to be come before X11 headers or libpng checks will get mad WRT setjmp version
-#ifdef CONFIG_LIBPNG
-#include <png.h>
-#endif
-
-#if defined CONFIG_XPM && !defined(CONFIG_GDK_PIXBUF_XLIB)
+#if defined CONFIG_XPM && !defined CONFIG_GDK_PIXBUF_XLIB && !defined CONFIG_IMLIB2
 
 #include <stdlib.h>
 #include <math.h>
 #include <errno.h>
+
+// include png before X11 or libpng checks will fail on setjmp version
+#ifdef CONFIG_LIBPNG
+#include <png.h>
+#endif
+
 #include "yimage.h"
 #include "yxapp.h"
 #include "ypointer.h"
@@ -22,7 +23,7 @@
 #include <setjmp.h>
 #endif
 
-#define ATH 55  /* highest alpha threshold that can show anti-aliased lines */
+#define ATH 10  /* highest alpha threshold that can show anti-aliased lines */
 
 struct Verbose {
     const bool verbose;
@@ -74,7 +75,7 @@ public:
     static ref<YImage> loadjpg(upath filename);
 #endif
     static ref<YImage> combine(XImage *xdraw, XImage *xmask);
-    static pstring detectImageType(upath filename);
+    static mstring detectImageType(upath filename);
 
     bool isBitmap() const { return fBitmap; }
     bool hasAlpha() const { return fImage ? fImage->depth == 32 : false; }
@@ -127,6 +128,10 @@ private:
     bool fBitmap;
 };
 
+const char* YImage::renderName() {
+    return "XImage";
+}
+
 bool YImage::supportsDepth(unsigned depth) {
     return depth == 32 || depth == xapp->depth();
 }
@@ -134,7 +139,7 @@ bool YImage::supportsDepth(unsigned depth) {
 ref<YImage> YImage::load(upath filename)
 {
     ref<YImage> image;
-    pstring ext(filename.getExtension().lower());
+    mstring ext(filename.getExtension().lower());
     bool unsup = false;
 
     if (ext.isEmpty())
@@ -163,19 +168,19 @@ ref<YImage> YImage::load(upath filename)
     else {
         unsup = true;
         if (ONCE)
-            warn(_("Unsupported file format: %s"), cstring(ext).c_str());
+            warn(_("Unsupported file format: %s"), ext.c_str());
     }
 
     if (image == null && !unsup)
-        fail(_("Could not load image \"%s\""), filename.string().c_str());
+        fail(_("Could not load image \"%s\""), filename.string());
     return image;
 }
 
-pstring YXImage::detectImageType(upath filename) {
+mstring YXImage::detectImageType(upath filename) {
      const int xpm = 9, png = 8, jpg = 4, len = max(xpm, png);
      char buf[len+1];
      memset(buf, 0, sizeof buf);
-     if (read_file(filename.string(), buf, sizeof buf) >= len) {
+     if (filereader(filename.string()).read_all(buf, sizeof buf) >= len) {
          if (0 == memcmp(buf, "/* XPM */", xpm)) {
              return ".xpm";
          }
@@ -186,11 +191,11 @@ pstring YXImage::detectImageType(upath filename) {
              return ".jpg";
          }
          else {
-             msg("Unknown image type: \"%s\".", filename.string().c_str());
+             msg("Unknown image type: \"%s\".", filename.string());
              return null;
          }
      }
-     fail(_("Could not load image \"%s\""), filename.string().c_str());
+     fail(_("Could not load image \"%s\""), filename.string());
      return null;
 }
 
@@ -205,7 +210,7 @@ ref <YImage> YXImage::loadxbm(upath filename)
 
     status = XReadBitmapFileData(filename.string(), &w, &h, (unsigned char **) &data, &x, &y);
     if (status != BitmapSuccess) {
-        tlog("ERROR: could not read pixmap file %s\n", filename.string().c_str());
+        tlog("ERROR: could not read pixmap file %s\n", filename.string());
         goto error;
     }
     xdraw = createBitmap(w, h, data);
@@ -306,15 +311,15 @@ ref<YImage> YXImage::loadpng(upath filename)
     FILE *f;
 
     if (!(f = fopen(filename.string(), "rb"))) {
-        tlog("could not open %s: %s\n", filename.string().c_str(), strerror(errno));
+        tlog("could not open %s: %s\n", filename.string(), strerror(errno));
         goto nofile;
     }
     if ((ret = fread(buf, 1, 8, f)) != 8) {
-        tlog("could not read PNG signature %s: %s\n", filename.string().c_str(), strerror(errno));
+        tlog("could not read PNG signature %s: %s\n", filename.string(), strerror(errno));
         goto noread;
     }
     if (png_sig_cmp(buf, 0, 8)) {
-        tlog("invalid PNG signature in %s\n", filename.string().c_str());
+        tlog("invalid PNG signature in %s\n", filename.string());
         goto noread;
     }
     if (!(png_ptr = png_create_read_struct(png_get_libpng_ver(NULL), NULL, NULL, NULL))) {
@@ -437,7 +442,7 @@ void YXImage::save(upath filename) {
     const char* error = "";
     if (savepng(filename, &error) == false) {
         fail("Cannot write YXImage %s: %s",
-            filename.string().c_str(), error);
+            filename.string(), error);
     }
 #endif
 }
@@ -530,7 +535,7 @@ ref<YImage> YXImage::loadjpg(upath filename)
     int row_stride;
     FILE* infile = filename.fopen("rb");
     if (infile == 0) {
-        fail("could not open %s", filename.string().c_str());
+        fail("could not open %s", filename.string());
         return yximage;
     }
 
@@ -1231,7 +1236,7 @@ void YXImage::composite(Graphics& g, int x, int y,
         dx = g.xorigin();
     }
     if (g.yorigin() > dy) {
-        if ((int) h <= g.xorigin() - dx) {
+        if ((int) h <= g.yorigin() - dy) {
             if (verbose)
             tlog("ERROR: coordinates out of bounds\n");
             return;
