@@ -1,8 +1,9 @@
 #include "config.h"
 #include "yxtray.h"
-#include "prefs.h"
-#include "wmoption.h"
 #include "ytimer.h"
+#include "yprefs.h"
+#include "default.h"
+#include "wmoption.h"
 #include <X11/Xutil.h>
 #include <X11/Xproto.h>
 #include <X11/extensions/Xcomposite.h>
@@ -127,7 +128,7 @@ YXTrayProxy::YXTrayProxy(const YAtom& atom, YXTray *tray):
     request_code(0)
 {
     singleton = this;
-    addStyle(wsNoExpose);
+    addStyle(wsNoExpose | wsToolTipping);
     setTitle("YXTrayProxy");
     if (isExternal()) {
         long orientation = SYSTEM_TRAY_ORIENTATION_HORZ;
@@ -586,6 +587,7 @@ YXTray::YXTray(YXTrayNotifier *notifier,
     addStyle(wsNoExpose);
     setTitle("YXTray");
     setParentRelative();
+    setSize(1, trayIconMaxHeight + fDrawBevel);
     fTrayProxy = new YXTrayProxy(atom, this);
     regainTrayWindows();
 }
@@ -793,7 +795,9 @@ void YXTray::relayout(bool enforced) {
     if (enforced == false)
         return;
 
-    for (IterType ec = fDocked.iterator(); ++ec; ) {
+    for (IterType ec = rightToLeft
+                     ? fDocked.reverseIterator()
+                     : fDocked.iterator(); ++ec; ) {
         if (false == ec->fVisible) {
             // skip
         }
@@ -844,7 +848,8 @@ void YXTray::trayUpdateGeometry(unsigned w, unsigned h, bool visible) {
     }
     MSG(("relayout %d %d : %d %d", w, h, width(), height()));
     if (w != width() || h != height()) {
-        fGeometry.setRect(x() + int(width()) - int(w), y(), w, h);
+        int ypos = (h == height()) ? y() : 1;
+        fGeometry.setRect(x() + int(width()) - int(w), ypos, w, h);
         setGeometry(fGeometry);
         if (visible)
             show();
